/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2018 Linagora
 *
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.pref;

import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;
import org.ow2.petals.cli.api.pref.exception.IncorrectPropertyValueException;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.http.EmbeddedHttpServerConfig;
import org.ow2.petals.cli.pref.exception.MissingPasswordException;
import org.ow2.petals.cli.pref.exception.MissingUserException;

public final class PreferencesImpl extends AbstractPreferencesImpl {

    /**
     * Name of the environment variable defining the preference file of Petals
     * CLI.
     */
    public static final String PREFERENCE_ENV_VAR = "PETALS_CLI_PREFS";

    /**
     * Name of the default resource used as preference file
     */
    public static final String PREFERENCE_FILE_NAME = "petals-cli.default";

    public static final String USERNAME_PROPERTY_SUFFIX = ".username";

    public static final String PASSWORD_PROPERTY_SUFFIX = ".password";

    public static final String PASSPHRASE_PROPERTY_SUFFIX = ".passphrase";

    /**
     * Property name identifying the HTTP listening port of the embedded HTTP server
     */
    public static final String EMBEDDED_HTTP_PORT = "embedded.http.port";

    /**
     * The default HTTP listening port (any free port) of the embedded HTTP server.
     */
    public static final int DEFAULT_EMBEDDED_HTTP_PORT = 0;

    /**
     * Embedded HTTP server parameters
     */
    private EmbeddedHttpServerConfig embeddedHttpServerParameters = null;

    public PreferencesImpl() {
        super(PREFERENCE_ENV_VAR, PREFERENCE_FILE_NAME);
    }

    @Override
    protected Collection<String> getRecognizedSuffixes() {
        final Collection<String> suffixes = new ArrayList<String>(super.getRecognizedSuffixes());
        suffixes.addAll(Arrays.asList(new String[] { USERNAME_PROPERTY_SUFFIX, PASSWORD_PROPERTY_SUFFIX,
                PASSPHRASE_PROPERTY_SUFFIX }));
        return Collections.unmodifiableCollection(suffixes);
    }

    /**
     * @return The preferences about the embedded HTTP server
     */
    public final EmbeddedHttpServerConfig getEmbeddedHttpServerParameters() {
        return this.embeddedHttpServerParameters;
    }

    @Override
    protected final ConnectionParameters createConnectionParameters(final Properties preferenceProperties,
            final String alias, final String host, final int port, final List<Exception> errors) {
        final String username = preferenceProperties.getProperty(alias + USERNAME_PROPERTY_SUFFIX);
        final String password = preferenceProperties.getProperty(alias + PASSWORD_PROPERTY_SUFFIX);
        final String passPhrase = preferenceProperties.getProperty(alias + PASSPHRASE_PROPERTY_SUFFIX);
        ConnectionParameters connectionParameters = null;
        try {
            if (host != null && port != -1 && username != null && password != null) {
                connectionParameters = new AuthenticatedConnectionParametersImpl(host, port, username, password,
                        passPhrase);
            } else if (host != null && port != -1 && username == null && password == null) {
                connectionParameters = new ConnectionParameters(host, port);
            } else if (username == null && password != null) {
                errors.add(new MissingUserException(alias));
            } else if (password == null && username != null) {
                errors.add(new MissingPasswordException(alias));
            }
        } catch (final UnknownHostException e) {
            errors.add(
                    new IncorrectPropertyValueException(alias + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, host, e));
        } catch (final UsernameMissingException e) {
            errors.add(new IncorrectPropertyValueException(alias + USERNAME_PROPERTY_SUFFIX, username, e));
        } catch (final PasswordMissingException e) {
            errors.add(new IncorrectPropertyValueException(alias + PASSWORD_PROPERTY_SUFFIX, password, e));
        } catch (final HostMissingException e) {
            errors.add(
                    new IncorrectPropertyValueException(alias + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, host, e));
        } catch (final InvalidPortException e) {
            errors.add(new IncorrectPropertyValueException(alias + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX,
                    String.valueOf(port), e));
        }
        return connectionParameters;
    }

    @Override
    protected void otherPreferenceInitialization(Properties preferenceProperties, List<Exception> errors) {
        super.otherPreferenceInitialization(preferenceProperties, errors);
        
        final String httpPortStr = preferenceProperties.getProperty(EMBEDDED_HTTP_PORT);
        if (httpPortStr != null && !httpPortStr.isEmpty()) {
            try {
                this.embeddedHttpServerParameters = new EmbeddedHttpServerConfig(Integer.parseInt(httpPortStr));
            } catch (final NumberFormatException e) {
                errors.add(new IncorrectPropertyValueException(EMBEDDED_HTTP_PORT, httpPortStr, e));
            }
        } else {
            this.embeddedHttpServerParameters = new EmbeddedHttpServerConfig(DEFAULT_EMBEDDED_HTTP_PORT);
        }
    }
}
