/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.UnknownHostException;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Arrays;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.command.AbstractCommand;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;

import jline.console.completer.FileNameCompleter;

public class CheckSSLHandshake extends AbstractCommand {

    public static final String HOST_SHORT_OPTION = "h";

    public static final String HOST_LONG_OPTION = "host";

    public static final String PORT_SHORT_OPTION = "p";

    public static final String PORT_LONG_OPTION = "port";

    public static final String KEYSTORE_FILE_SHORT_OPTION = "k";

    public static final String KEYSTORE_FILE_LONG_OPTION = "keystore-file";

    public static final String KEYSTORE_PASS_SHORT_OPTION = "u";

    public static final String KEYSTORE_PASS_LONG_OPTION = "keystore-passphrase";

    public static final String TRUST_ALL_CERTS_LONG_OPTION = "trust-all-certificates";

    protected static final Option HOST_OPTION = Option.builder(HOST_SHORT_OPTION).numberOfArgs(1).argName("host")
            .longOpt(HOST_LONG_OPTION).desc("Hostname to connect. Default value used: localhost").build();

    protected static final Option PORT_OPTION = Option.builder(PORT_SHORT_OPTION).numberOfArgs(1).argName("port")
            .longOpt(PORT_LONG_OPTION).desc("Port on host to connect. Default value used: 443 (HTTPS)").build();

    protected static final Option KEYSTORE_FILE_OPTION = Option.builder(KEYSTORE_FILE_SHORT_OPTION).numberOfArgs(1)
            .argName("keystore-file").longOpt(KEYSTORE_FILE_LONG_OPTION)
            .desc("Keystore file containing certificates. Default keystore: the JVM one").build();

    protected static final Option KEYSTORE_PASS_OPTION = Option.builder(KEYSTORE_PASS_SHORT_OPTION).numberOfArgs(1)
            .argName("keystore-file").longOpt(KEYSTORE_PASS_LONG_OPTION)
            .desc("Passphrase to open the keystore containing certificates.").build();

    protected static final Option TRUST_ALL_CERTS_OPTION = Option.builder().numberOfArgs(0)
            .longOpt(TRUST_ALL_CERTS_LONG_OPTION).desc("Trust all certificates. Certificate validation is disabled.")
            .build();

    protected static final String CHECK_OK_MESSAGE = "No errors, certificate configuration is ok !";

    private static final String DEFAULT_HOST = "localhost";

    private static final int DEFAULT_PORT = 443;

    public CheckSSLHandshake() {
        super("check-ssl-handshake");
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Check SSL handshake to a given server.");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        final FileNameCompleter c = new FileNameCompleter();
        this.completers.put(KEYSTORE_FILE_SHORT_OPTION, c);
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {

        final CommandLineParser parser = new DefaultParser();
        try {

            final CommandLine cli = parser.parse(this.getOptions(), args);

            final String hostname;
            if (cli.hasOption(HOST_SHORT_OPTION)) {
                if (!this.checkArguments(args, 2, 8)) {
                    throw new CommandTooManyArgumentsException(this, args);
                }
                hostname = cli.getOptionValue(HOST_SHORT_OPTION);
            } else {
                hostname = DEFAULT_HOST;
            }

            final int port;
            if (cli.hasOption(PORT_SHORT_OPTION)) {
                if (!this.checkArguments(args, 2, 8)) {
                    throw new CommandTooManyArgumentsException(this, args);
                }
                final String portStr = cli.getOptionValue(PORT_SHORT_OPTION);
                try {
                    port = Integer.parseInt(portStr);
                } catch (final NumberFormatException e) {
                    throw new CommandInvalidArgumentException(this, PORT_OPTION, portStr, e);
                }
            } else {
                port = DEFAULT_PORT;
            }

            final File keystoreFile;
            final String keystorePass;
            if (cli.hasOption(KEYSTORE_FILE_SHORT_OPTION)) {
                if (!this.checkArguments(args, 2, 8)) {
                    throw new CommandTooManyArgumentsException(this, args);
                }
                keystoreFile = new File(cli.getOptionValue(KEYSTORE_FILE_SHORT_OPTION));

                if (cli.hasOption(KEYSTORE_PASS_SHORT_OPTION)) {
                    keystorePass = cli.getOptionValue(KEYSTORE_PASS_SHORT_OPTION);
                } else {
                    throw new CommandMissingOptionsException(this, Arrays.asList(KEYSTORE_PASS_SHORT_OPTION));
                }
            } else if (cli.hasOption(KEYSTORE_PASS_SHORT_OPTION)) {
                throw new CommandMissingOptionsException(this, Arrays.asList(KEYSTORE_FILE_SHORT_OPTION));
            } else {
                keystoreFile = null;
                keystorePass = null;
            }

            final boolean trustAllCerts;
            if (cli.hasOption(TRUST_ALL_CERTS_LONG_OPTION)) {
                if (keystoreFile != null) {
                    throw new CommandTooManyArgumentsException(this,
                            new String[] { TRUST_ALL_CERTS_LONG_OPTION, KEYSTORE_FILE_SHORT_OPTION });
                } else {
                    trustAllCerts = true;
                }
            } else {
                trustAllCerts = false;
            }

            this.tryConnection(hostname, port, keystoreFile, keystorePass, trustAllCerts);

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final DuplicatedServiceException | MissingServiceException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(HOST_OPTION);
        options.addOption(PORT_OPTION);
        options.addOption(KEYSTORE_FILE_OPTION);
        options.addOption(KEYSTORE_PASS_OPTION);
        options.addOption(TRUST_ALL_CERTS_OPTION);
        return options;
    }

    @Override
    public boolean isConnectionRequired() {
        return false;
    }

    private void tryConnection(final String hostname, final int port, final File keystoreFile,
            final String keystorePass, final boolean trustAllCerts) throws CommandException {

        final SSLSocketFactory factory;
        try {
            final SSLContext sslContext = SSLContext.getInstance("TLS");
            final TrustManager[] trustManagers = this.loadOrCreateTrustManagers(keystoreFile, keystorePass,
                    trustAllCerts);
            sslContext.init(null, trustManagers, null);
            factory = sslContext.getSocketFactory();
        } catch (final GeneralSecurityException e) {
            throw new CommandException(this, e);
        }

        this.getShell().getPrintStream().println(String.format("Opening connection to %s:%d...", hostname, port));
        try (final SSLSocket socket = (SSLSocket) factory.createSocket(hostname, port)) {
            socket.setSoTimeout(10000);

            this.executeSSLHandshake(socket);

        } catch (final UnknownHostException e) {
            throw new CommandInvalidArgumentException(this, HOST_OPTION, hostname, e);
        } catch (final IllegalArgumentException e) {
            throw new CommandInvalidArgumentException(this, PORT_OPTION, String.valueOf(port), e);
        } catch (final IOException e) {
            throw new CommandException(this, e);
        }
    }

    private TrustManager[] loadOrCreateTrustManagers(final File keystoreFile, final String keystorePass,
            final boolean trustAllCerts) throws CommandException {

        if (trustAllCerts) {
            return new TrustManager[] { new X509TrustManager() {

                @Override
                public void checkClientTrusted(final X509Certificate[] chain, final String authType)
                        throws CertificateException {
                    // NOP
                }

                @Override
                public void checkServerTrusted(final X509Certificate[] chain, final String authType)
                        throws CertificateException {
                    // NOP: All servers are accepted
                }

                @Override
                public X509Certificate[] getAcceptedIssuers() {
                    return new X509Certificate[0];
                }
            } };
        } else if (keystoreFile == null) {
            return null;
        } else {

            try {
                final KeyStore keystore = this.loadKeyStore(keystoreFile, keystorePass);

                final TrustManagerFactory tmFactory = TrustManagerFactory
                        .getInstance(TrustManagerFactory.getDefaultAlgorithm());
                tmFactory.init(keystore);
                return tmFactory.getTrustManagers();
            } catch (final GeneralSecurityException e) {
                throw new CommandException(this, e);
            }
        }

    }

    private KeyStore loadKeyStore(final File keystoreFile, final String keystorePass)
            throws CommandInvalidArgumentException {

        try (final InputStream fisKeystore = new FileInputStream(keystoreFile)) {
            final KeyStore keystore = KeyStore.getInstance(KeyStore.getDefaultType());
            keystore.load(fisKeystore, keystorePass.toCharArray());
            return keystore;
        } catch (final IOException | GeneralSecurityException e) {
            throw new CommandInvalidArgumentException(this, KEYSTORE_FILE_OPTION, keystoreFile.getAbsolutePath(), e);
        }
    }

    private void executeSSLHandshake(final SSLSocket socket) throws CommandException {
        try {
            this.getShell().getPrintStream().println("Starting SSL handshake...");

            socket.startHandshake();

            this.getShell().getPrintStream().println();
            this.getShell().getPrintStream().println(CHECK_OK_MESSAGE);
        } catch (final IOException e) {
            this.getShell().getErrorStream().println();
            e.printStackTrace(this.getShell().getErrorStream());
            throw new CommandException(this, e);
        }

    }
}
