/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Component.ComponentType;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;

import com.ebmwebsourcing.easycommons.lang.CollectionHelper;
import com.ebmwebsourcing.easycommons.lang.CollectionHelper.Filter;

/**
 * This command list JBI artifacts deployed on petals ESB.
 * @author Sebastien Andre - EBM WebSourcing
 */
public class ListArtifacts extends AbstractPetalsCliCommand implements ArtifactTypeBasedCommand {

    static final String REGEX_IDENTIFIER_SHORT_OPTION = "p";

    static final String REGEX_IDENTIFIER_LONG_OPTION = "identifier-pattern";

    private static final Option REGEX_IDENTIFIER_OPTION = Option.builder(REGEX_IDENTIFIER_SHORT_OPTION).required(false)
            .longOpt(REGEX_IDENTIFIER_LONG_OPTION)
            .desc("<identifier-pattern> is an optional RegExp pattern to filter the content of the returned list. All the returned JBI artifacts must have a JBI identifier matching this pattern.")
            .numberOfArgs(1).argName("identifier-pattern").build();

    static final String SL_VERSION_REGEX_SHORT_OPTION = "v";

    static final String SL_VERSION_REGEX_LONG_OPTION = "version-pattern";

    private static final Option SL_VERSION_REGEX_OPTION = Option.builder(SL_VERSION_REGEX_SHORT_OPTION).required(false)
            .longOpt(SL_VERSION_REGEX_LONG_OPTION)
            .desc("<version-pattern> is an optional RegExp pattern to filter the content of the returned list. All the returned JBI shared libraries must have a version matching this pattern.")
            .numberOfArgs(1).argName("version-pattern").build();

    private final static String SEPARATOR = " ";

    /**
     * An artifact name comparator
     */
    protected final Comparator<Artifact> artifactNameComparator = new Comparator<Artifact>() {
        @Override
        public int compare(final Artifact artifact1, final Artifact artifact2) {
            final int nameComparaison = artifact1.getName().compareTo(artifact2.getName());
            if (nameComparaison == 0) {
                final String version1 = artifact1.getVersion();
                final String version2 = artifact2.getVersion();
                if (version1 == null) {
                    if (version2 == null) {
                        return 0;
                    } else {
                        return 1;
                    }
                } else {
                    if (version2 == null) {
                        return -1;
                    } else {
                        return version1.compareTo(version2);
                    }
                }
            } else {
                return nameComparaison;
            }
        }
    };

    private final DefaultParser clParser;

    public ListArtifacts() {
        super("list");
        this.clParser = new DefaultParser();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("List JBI artifacts name and current status");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        this.completers.put(TYPE_SHORT_OPTION, ARTIFACT_TYPE_COMPLETER);
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        try {
            final List<Filter<Artifact>> filters = new ArrayList<>();
            final CommandLine cli = this.clParser.parse(getOptions(), args);
            if (cli.getArgs().length > 0) {
                throw new CommandBadArgumentNumberException(this);
            }

            if (cli.hasOption(TYPE_SHORT_OPTION)) {
                filters.add(new TypeFilter(cli.getOptionValue(TYPE_SHORT_OPTION)));
            }
            if (cli.hasOption(REGEX_IDENTIFIER_SHORT_OPTION)) {
                filters.add(new ArtifactIdentifierRegexFilter(cli.getOptionValue(REGEX_IDENTIFIER_SHORT_OPTION)));
            }
            if (cli.hasOption(SL_VERSION_REGEX_SHORT_OPTION)) {
                filters.add(new SharedLibraryVersionRegexFilter(cli.getOptionValue(SL_VERSION_REGEX_SHORT_OPTION)));
            }

            final ArtifactAdministration artifactAdministration = getAdmin().newArtifactAdministration();
            final List<Artifact> artifacts = artifactAdministration.listArtifacts();

            final StringBuilder out = new StringBuilder();
            for (final Artifact artifact : sortArtifacts(artifacts, filters)) {
                if (SharedLibrary.TYPE.equals(artifact.getType())) {
                    out.append(artifact.getName()).append('(').append(artifact.getVersion()).append(')');
                } else {
                    out.append(artifact.getName());
                }
                if (artifact instanceof ArtifactState) {
                    out.append(SEPARATOR);
                    out.append(((ArtifactState) artifact).getState().toString());
                }
                out.append(SEPARATOR);
                out.append(artifact.getType());
                out.append("\n");
            }
            if (!artifacts.isEmpty()) {
                this.getShell().getPrintStream().print(out);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ArtifactAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final DuplicatedServiceException e) {
            throw new CommandException(this, e);
        } catch (final MissingServiceException e) {
            throw new CommandException(this, e);
        }
    }

    /**
     *
     * @param artifacts
     * @return
     */
    private List<Artifact> sortArtifacts(final List<Artifact> artifacts, final List<Filter<Artifact>> filters) {
        final Collection<SharedLibrary> sls = new TreeSet<>(this.artifactNameComparator);
        final Collection<Component> bcs = new TreeSet<>(this.artifactNameComparator);
        final Collection<Component> ses = new TreeSet<>(this.artifactNameComparator);
        final Collection<ServiceAssembly> sas = new TreeSet<>(this.artifactNameComparator);
        final Collection<ServiceUnit> sus = new TreeSet<>(this.artifactNameComparator);

        for (final Artifact artifact : artifacts) {
            if (artifact instanceof SharedLibrary) {
                sls.add((SharedLibrary) artifact);
            } else if (ComponentType.BC.toString().equals(artifact.getType())) {
                bcs.add((Component) artifact);
            } else if (ComponentType.SE.toString().equals(artifact.getType())) {
                ses.add((Component) artifact);
            } else if (artifact instanceof ServiceAssembly) {
                sas.add((ServiceAssembly) artifact);
                sus.addAll(((ServiceAssembly) artifact).getServiceUnits());
            }
        }

        final List<Artifact> sortedArtifacts = new ArrayList<Artifact>(sls);
        sortedArtifacts.addAll(bcs);
        sortedArtifacts.addAll(ses);
        sortedArtifacts.addAll(sas);
        sortedArtifacts.addAll(sus);

        if (filters != null && !filters.isEmpty()) {
            CollectionHelper.filter(sortedArtifacts, filters);
        }

        return sortedArtifacts;
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(TYPE_OPTION);
        options.addOption(REGEX_IDENTIFIER_OPTION);
        options.addOption(SL_VERSION_REGEX_OPTION);
        return options;
    }

    private static class TypeFilter implements Filter<Artifact> {
        private final String type;

        public TypeFilter(final String type) {
            this.type = type;
        }

        @Override
        public boolean accept(final Artifact a) {
            if (this.type == null) {
                return true;
            } else {
                return a.getType().equals(this.type);
            }
        }
    }

    /**
     * Implements strict regex filter on artifact identifier
     * 
     * @author "Mathieu CARROLLE - EBM WebSourcing"
     * 
     */
    private static class ArtifactIdentifierRegexFilter implements Filter<Artifact> {
        private final Pattern pattern;

        public ArtifactIdentifierRegexFilter(final String regex) {
            this.pattern = Pattern.compile(regex);
        }

        @Override
        public boolean accept(final Artifact a) {
            final Matcher m = this.pattern.matcher(a.getName());
            return m.matches();
        }
    }

    /**
     * Implements strict regex filter on shared library version
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    private static class SharedLibraryVersionRegexFilter implements Filter<Artifact> {
        private final Pattern pattern;

        public SharedLibraryVersionRegexFilter(final String regex) {
            this.pattern = Pattern.compile(regex);
        }

        @Override
        public boolean accept(final Artifact a) {
            if (SharedLibrary.TYPE.equals(a.getType())) {
                final Matcher m = this.pattern.matcher(a.getVersion());
                return m.matches();
            } else {
                return true;
            }
        }
    }

}
