/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.io.PrintStream;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.EndpointDirectoryAdministration;
import org.ow2.petals.admin.api.exception.EndpointDirectoryAdministrationException;
import org.ow2.petals.admin.endpoint.Endpoint;
import org.ow2.petals.admin.endpoint.EndpointDirectoryView;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.shell.completer.ContainerNameCompleter;

import jline.console.completer.Completer;

/**
 * This command 'endpoint-list' browsing the endpoint directory of the topology of the current Petals ESB container.
 * 
 * @author Sebastien Andre - EBM WebSourcing
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class ListEndpoints extends AbstractPetalsCliCommand {

    private static final String DEFAULT_REGEX_OPTION_VALUE = ".*";

    /**
     * The container name regular expression option
     */
    private static final String CONTAINER_FILTER_SHORT_OPTION = "c";

    private static final String CONTAINER_FILTER_LONG_OPTION = "container";

    private static final String CONTAINER_FILTER_ARG_NAME = "container";

    private static final Option CONTAINER_FILTER_OPTION = Option.builder(CONTAINER_FILTER_SHORT_OPTION).numberOfArgs(1)
            .argName(CONTAINER_FILTER_ARG_NAME).longOpt(CONTAINER_FILTER_LONG_OPTION)
            .desc("Container name regular expression.").build();

    /**
     * The endpoint name regular expression option
     */
    private static final String ENDPOINT_NAME_SHORT_OPTION = "e";
    private static final String ENDPOINT_NAME_LONG_OPTION = "endpoint";
    private static final String ENDPOINT_NAME_ARG_NAME = "endpoint";

    private static final Option ENDPOINT_NAME_OPTION = Option.builder(ENDPOINT_NAME_SHORT_OPTION).numberOfArgs(1)
            .argName(ENDPOINT_NAME_ARG_NAME).longOpt(ENDPOINT_NAME_LONG_OPTION)
            .desc("Endpoint name regular expression.").build();

    /**
     * The service name regular expression option
     */
    private static final String SERVICE_NAME_SHORT_OPTION = "s";
    private static final String SERVICE_NAME_LONG_OPTION = "service";
    private static final String SERVICE_NAME_ARG_NAME = "service";

    private static final Option SERVICE_NAME_OPTION = Option.builder(SERVICE_NAME_SHORT_OPTION).numberOfArgs(1)
            .argName(SERVICE_NAME_ARG_NAME).longOpt(SERVICE_NAME_LONG_OPTION).desc("Service name regular expression.")
            .build();

    /**
     * The interface name regular expression option
     */
    private static final String INTERFACE_NAME_SHORT_OPTION = "i";
    private static final String INTERFACE_NAME_LONG_OPTION = "interface";
    private static final String INTERFACE_NAME_ARG_NAME = "interface";

    private static final Option INTERFACE_NAME_OPTION = Option.builder(INTERFACE_NAME_SHORT_OPTION).numberOfArgs(1)
            .argName(INTERFACE_NAME_ARG_NAME).longOpt(INTERFACE_NAME_LONG_OPTION)
            .desc("Interface name regular expression.").build();

    /**
     * The flag selecting the part 'endpoints' for displaying
     */
    protected static final String OUTPUT_ENDPOINTS_LONG_OPTION = "output-endpoints";

    private static final Option OUTPUT_ENDPOINTS_OPTION = Option.builder().hasArg(false).required(false)
            .longOpt(OUTPUT_ENDPOINTS_LONG_OPTION).desc("Output selector about endpoints.").build();

    /**
     * The flag selecting the part 'services' for displaying
     */
    protected static final String OUTPUT_SERVICES_LONG_OPTION = "output-services";

    private static final Option OUTPUT_SERVICES_OPTION = Option.builder().hasArg(false).required(false)
            .longOpt(OUTPUT_SERVICES_LONG_OPTION).desc("Output selector about services.").build();

    /**
     * The flag selecting the part 'interfaces' for displaying
     */
    protected static final String OUTPUT_INTERFACES_LONG_OPTION = "output-interfaces";

    private static final Option OUTPUT_INTERFACES_OPTION = Option.builder().hasArg(false).required(false)
            .longOpt(OUTPUT_INTERFACES_LONG_OPTION).desc("Output selector about interfaces.").build();

    public ListEndpoints() {
        super("endpoint-list");
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("List the entries of the endpoint directory");
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        try {
            final CommandLineParser parser = new DefaultParser();
            final CommandLine cmd = parser.parse(this.getOptions(), args);

            final String containerNameRegex = getRegexOption(cmd, CONTAINER_FILTER_SHORT_OPTION);
            final String endpointNameRegex = getRegexOption(cmd, ENDPOINT_NAME_SHORT_OPTION);
            final String serviceNameRegex = getRegexOption(cmd, SERVICE_NAME_SHORT_OPTION);
            final String interfaceNameRegex = getRegexOption(cmd, INTERFACE_NAME_SHORT_OPTION);

            final EndpointDirectoryAdministration endpointDirectoryAdministration = getAdmin()
                    .newEndpointDirectoryAdministration();
            final EndpointDirectoryView endpointDirectoryView = endpointDirectoryAdministration
                    .getEndpointDirectoryContent(containerNameRegex, endpointNameRegex, serviceNameRegex,
                            interfaceNameRegex);
            
            final boolean hasAnOutputFlag = cmd.hasOption(OUTPUT_ENDPOINTS_LONG_OPTION)
                    || cmd.hasOption(OUTPUT_SERVICES_LONG_OPTION) || cmd.hasOption(OUTPUT_INTERFACES_LONG_OPTION);
            final boolean isOutputEndpoints = cmd.hasOption(OUTPUT_ENDPOINTS_LONG_OPTION) || !hasAnOutputFlag;
            final boolean isOutputServices = cmd.hasOption(OUTPUT_SERVICES_LONG_OPTION) || !hasAnOutputFlag;
            final boolean isOutputInterfaces = cmd.hasOption(OUTPUT_INTERFACES_LONG_OPTION) || !hasAnOutputFlag;

            final StringBuilder endpointDirectoryContent = new StringBuilder();
            if (isOutputEndpoints) {
                if (isOutputServices || isOutputInterfaces) {
                    endpointDirectoryContent.append("Endpoints:\n");
                }
                final List<Endpoint> endpoints = endpointDirectoryView.getAllEndpoints();
                for (final Endpoint endpoint : endpoints) {
                    final String endpointInformation = String.format("%s: %s,%s,%s%n", endpoint.getEndpointName(),
                            endpoint.getContainerName(), endpoint.getComponentName(), endpoint.getEndpointType());
                    endpointDirectoryContent.append(endpointInformation);
                }
            }

            if (isOutputServices) {
                if (isOutputEndpoints || isOutputInterfaces) {
                    endpointDirectoryContent.append("Services:\n");
                }
                final Map<String, List<Endpoint>> endpointsByServiceName = endpointDirectoryView
                        .getListOfEndpointsByServiceName();
                for (final Entry<String, List<Endpoint>> entry : endpointsByServiceName.entrySet()) {
                    final String serviceName = entry.getKey();
                    endpointDirectoryContent.append(serviceName + ":\n");
                    int idx = 0;
                    for (final Endpoint endpoint : entry.getValue()) {
                        endpointDirectoryContent.append(endpoint.getEndpointName()
                                + (++idx == entry.getValue().size() ? "" : " "));
                    }
                    endpointDirectoryContent.append("\n");
                }
            }

            if (isOutputInterfaces) {
                if (isOutputEndpoints || isOutputServices) {
                    endpointDirectoryContent.append("Interfaces:\n");
                }
                final Map<String, List<Endpoint>> endpointsByInterfaceName = endpointDirectoryView
                        .getListOfEndpointsByInterfaceName();
                for (final Entry<String, List<Endpoint>> entry : endpointsByInterfaceName.entrySet()) {
                    endpointDirectoryContent.append(entry.getKey() + ":\n");
                    int idx = 0;
                    for (final Endpoint endpoint : entry.getValue()) {
                        endpointDirectoryContent.append(endpoint.getEndpointName()
                                + (++idx == entry.getValue().size() ? "" : " "));
                    }
                    endpointDirectoryContent.append("\n");
                }
            }

            final PrintStream out = this.getShell().getPrintStream();
            out.print(endpointDirectoryContent);
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final EndpointDirectoryAdministrationException e) {
            throw new CommandException(this, e);
        }
    }

    private static final String getRegexOption(final CommandLine cmd, final String option) {
        final String endpointNameRegex;
        if (cmd.hasOption(option)) {
            endpointNameRegex = cmd.getOptionValue(option);
        } else {
            endpointNameRegex = DEFAULT_REGEX_OPTION_VALUE;
        }
        return endpointNameRegex;
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(CONTAINER_FILTER_OPTION);
        options.addOption(ENDPOINT_NAME_OPTION);
        options.addOption(SERVICE_NAME_OPTION);
        options.addOption(INTERFACE_NAME_OPTION);
        options.addOption(OUTPUT_ENDPOINTS_OPTION);
        options.addOption(OUTPUT_SERVICES_OPTION);
        options.addOption(OUTPUT_INTERFACES_OPTION);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        this.completers.put(CONTAINER_FILTER_SHORT_OPTION,
                new ContainerNameCompleter(getAdmin().newContainerAdministration()));
        return this.completers;
    }
}
