/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.ow2.petals.cli.Constants.CommonOption.URL_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;

import java.io.PrintStream;
import java.net.MalformedURLException;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.shell.completer.AllArtifactNameCompleter;
import org.ow2.petals.cli.shell.util.Utils;

import jline.console.completer.Completer;
import jline.console.completer.FileNameCompleter;

/**
 * This command shows information about remote JBI artifacts.
 * @author Sebastien Andre - EBM WebSourcing
 */
public class Show extends AbstractPetalsCliCommand implements ArtifactBasedCommand, ArtifactVersionBasedCommand {

    static final String UNKNOWN_VERSION = "Not available";

    private static final String EXTENDED_INFORMATION_LBL = "Extended information:";

    private static final String EXTENDED_SHORT_OPTION = "e";

    private static final String EXTENDED_LONG_OPTION = "extended";

    protected static final String SERVICEUNITS_LBL = "ServiceUnits";

    protected static final String IDENTIFIER_LBL = "Identifier";

    protected static final String TYPE_LBL = "Type";

    protected static final String STATE_LBL = "State";

    protected static final String VERSION_LBL = "Version";

    protected static final String SEPARATOR = ":";

    private final CommandLineParser clParser;

    public Show() {
        super();
        this.clParser = new DefaultParser();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Print informations about a JBI artifact");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        this.completers.put(URL_SHORT_OPTION, new FileNameCompleter());
        this.completers.put(TYPE_SHORT_OPTION, ARTIFACT_TYPE_COMPLETER);
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        ArtifactAdministration artifactAdministration = getAdmin().newArtifactAdministration();
        try {
            Artifact artifact = null;
            final CommandLine cli = this.clParser.parse(getOptions(), args, false);
            final boolean extended = cli.hasOption(EXTENDED_SHORT_OPTION);
            if (cli.hasOption(URL_SHORT_OPTION)) {
                if (extended) {
                    if (!this.checkArguments(args, 3, 3)) {
                        throw new CommandTooManyArgumentsException(this, args);
                    }
                } else {
                    if (!this.checkArguments(args, 2, 2)) {
                        throw new CommandTooManyArgumentsException(this, args);
                    }
                }
                artifact = artifactAdministration.getArtifactInfo(Utils.toURL(cli
                        .getOptionValue(URL_SHORT_OPTION)));
            } else if (cli.hasOption(ARTIFACT_SHORT_OPTION)) {
                if (extended) {
                    if (!this.checkArguments(args, 3, 6)) {
                        throw new CommandTooManyArgumentsException(this, args);
                    }
                } else {
                    if (!this.checkArguments(args, 2, 5)) {
                        throw new CommandTooManyArgumentsException(this, args);
                    }
                }
                artifact = artifactAdministration.getArtifactInfo(cli.getOptionValue(TYPE_SHORT_OPTION),
                        cli.getOptionValue(ARTIFACT_SHORT_OPTION), cli.getOptionValue(VERSION_SHORT_OPTION));
            } else {
                throw new CommandBadArgumentNumberException(this);
            }
            final PrintStream ps = this.getShell().getPrintStream();
            if (artifact instanceof Component) {
                printComponentInformation(ps, (Component) artifact, extended);
            } else if (artifact instanceof ServiceAssembly) {
                printServiceAssemblyInformation(ps, (ServiceAssembly) artifact, extended);
            } else if (artifact instanceof SharedLibrary) {
                printSharedLibraryInformation(ps, (SharedLibrary) artifact, extended);
            } else {
                throw new CommandException(this, "Unknown artifact type.");
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ArtifactAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final MalformedURLException e) {
            throw new CommandException(this, e);
        }
    }

    private void printSharedLibraryInformation(final PrintStream ps, final SharedLibrary sl, final boolean extended) {
        ps.println(IDENTIFIER_LBL + SEPARATOR + sl.getName());
        ps.println(VERSION_LBL + SEPARATOR
                + (sl.getVersion() == null ? UNKNOWN_VERSION : sl.getVersion()));
    }

    private void printComponentInformation(final PrintStream ps, final Component c, final boolean extended) {
        ps.println(IDENTIFIER_LBL + SEPARATOR + c.getName());
        ps.println(TYPE_LBL + SEPARATOR + c.getComponentType());
        ps.println(VERSION_LBL + SEPARATOR
                + (c.getVersion() == null ? UNKNOWN_VERSION : c.getVersion()));
        ps.println(STATE_LBL + SEPARATOR + c.getState());
    }

    private void printServiceAssemblyInformation(final PrintStream ps, final ServiceAssembly sa, final boolean extended) {
        ps.println(IDENTIFIER_LBL + SEPARATOR + sa.getName());
        ps.println(VERSION_LBL + SEPARATOR
                + (sa.getVersion() == null ? UNKNOWN_VERSION : sa.getVersion()));
        ps.println(STATE_LBL + SEPARATOR + sa.getState());
        final StringBuilder b = new StringBuilder();
        final List<ServiceUnit> sus = sa.getServiceUnits();
        if (extended) {
            ps.println(EXTENDED_INFORMATION_LBL);
            for (ServiceUnit serviceUnit : sus) {
                ps.println("\t" + serviceUnit.getName() + " " + serviceUnit.getTargetComponent());
            }
        } else {
            for (int i = 0; i < sus.size(); i++) {
                b.append(sus.get(i).getName());
                if (i != sus.size() - 1) {
                    b.append(" ");
                }
            }
            ps.println(SERVICEUNITS_LBL + SEPARATOR + b.toString());
        }
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(ARTIFACT_OPTION);
        options.addOption(TYPE_OPTION);
        options.addOption(VERSION_OPTION);
        options.addOption(URL_OPTION);
        options.addOption(Option.builder(EXTENDED_SHORT_OPTION).longOpt(EXTENDED_LONG_OPTION)
                .desc("Display extended information.").hasArg(false).build());
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        this.completers.put(ARTIFACT_SHORT_OPTION, new AllArtifactNameCompleter(
                getAdmin().newArtifactAdministration()));
        return this.completers;
    }

}
