/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli;

import static org.hamcrest.Matchers.allOf;
import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.startsWith;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.junit.ContainerAdministrationMock;
import org.ow2.petals.admin.junit.PetalsAdministrationMock;
import org.ow2.petals.admin.junit.utils.SystemInfoUtils;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.base.junit.shell.PetalsInteractiveCliUtils;
import org.ow2.petals.cli.command.Help;
import org.ow2.petals.cli.shell.ShellFactory;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.cli.shell.command.Version;
import org.ow2.petals.cli.shell.extension.PetalsAdminShellExtension;

import com.jayway.awaitility.Awaitility;
import com.jayway.awaitility.Duration;

/**
 * Unit test of {@link Main}.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class MainTest extends AbstractCliTest {

    @Rule
    public final TemporaryFolder tempFolder = new TemporaryFolder();

    /**
     * <p>
     * Check the main entry point against a connection establishment on a unresolvable hostname in command line mode,
     * ie. a command line containing:
     * </p>
     * <ul>
     * <li>the command mode without connection arguments,</li>
     * <li>the operand flag,</li>
     * <li>a command requiring a connection,</li>
     * <li>a connection to a hostname unresolvable.</li>
     * </ul>
     * <p>
     * as following:<p>
     * <pre>
     * {@code > ./petals-cli.sh -h unresolvable-host -n 7700 -p petals -u petals -c -- list}</pre>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error about the unresovable hostname,</li>
     * <li>return code 1.</li>
     * </ul>
     */
    @Test
    public final void testRun_Command_ConnectionToUnresolvableHostname() throws Exception {

        final Command helpCommand = new Help();
        final String unresolvableHostname = "unresolvable-host";
        runCliBlocking(1, 60, "-" + Connect.HOST_SHORT_OPTION, unresolvableHostname, "-" + Connect.PORT_SHORT_OPTION,
                "7700", "-" + Connect.USER_SHORT_OPTION, "petals", "-" + Connect.PWD_SHORT_OPTION, "petals",
                "-" + ShellFactory.COMMAND_SHORT_OPTION, "--", helpCommand.getName());

        assertFalse("Usage is not print", systemOut.getLog().isEmpty());
        assertTrue("Unresolvable hostname is missing on the error message",
                systemErr.getLog().contains(unresolvableHostname));

    }

    /**
     * <p>
     * Test the correct disconnection when:
     * </p>
     * <ul>
     * <li>a connection is established using the yes flag,</li>
     * <li>a command requiring the connection is executed (ex: 'version' that requires a connection in the
     * implementation of the Petals Admin API mock),</li>
     * <li>the command 'disconnect' or 'exit' is not executed before to end the Petals CLI script,</li>
     * <li>using the inlined mode.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>No error occurs,</li>
     * <li>The connection is established on 'connect' from preferences,</li>
     * <li>The connection is correctly disconnected.</li>
     * </ul>
     */
    @Test
    public void testRun_Inlined_Disconnection_00() throws Exception {

        PetalsAdminShellExtension admin = runCli(createBasicPreferences("localhost", 7700, "petals", "petals", null),
                "-h",
                "localhost", "-n", "7700", "-u", "petals", "-p", "petals", "-");

        systemIn.writeLine("connect -y");
        systemIn.writeLine("version");
        systemIn.writeLine("exit");

        waitThreadOrFail();

        assertTrue("Connection not established", systemOut.getLog().contains(SystemInfoUtils.SYSTEM_INFO_EXAMPLE));
        assertTrue("An error occurs", systemErr.getLog().isEmpty());
        assertFalse("Connection not disconnected", admin.getAdmin().isConnected());

    }

    /**
     * <p>
     * Test the correct disconnection when:
     * </p>
     * <ul>
     * <li>a connection is established using the yes flag,</li>
     * <li>a command requiring the connection is executed (ex: 'version' that requires a connection in the
     * implementation of the Petals Admin API mock),</li>
     * <li>the command 'disconnect' or 'exit' is not executed before to end the Petals CLI script,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>No error occurs,</li>
     * <li>The connection is established on 'connect',</li>
     * <li>The connection is correctly disconnected.</li>
     * </ul>
     */
    @Test
    public void testRun_File_Disconnection_00() throws Exception {

        final Command commandConnect = new Connect();
        final Command commandVersion = new Version();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        try (final OutputStream osScript = new FileOutputStream(petalsCliScript);
                final Writer pwScript = new OutputStreamWriter(osScript)) {
            pwScript.write(commandConnect.getName());
            pwScript.write(" -y\n");
            pwScript.write(commandVersion.getName());
            pwScript.write('\n');
        }

        PetalsAdministration admin = runCliBlocking(
                createBasicPreferences("localhost", 7700, "petals", "petals", null),
                "-" + ShellFactory.FILE_LONG_OPTION, petalsCliScript.getAbsolutePath());

        assertTrue("Connection not established", systemOut.getLog().contains(SystemInfoUtils.SYSTEM_INFO_EXAMPLE));
        assertTrue("An error occurs", systemErr.getLog().isEmpty());
        assertFalse("Connection not disconnected", admin.isConnected());

    }

    /**
     * <p>
     * Test the correct disconnection when:
     * </p>
     * <ul>
     * <li>using the command line shell,</li>
     * <li>a automatic connection is established,</li>
     * <li>a command requiring the connection is executed (ex: 'version' that requires a connection in the
     * implementation of the Petals Admin API mock).</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>No error occurs, connection parameters are read from the preference files</li>
     * <li>The connection is established before to execute 'version',</li>
     * <li>The connection is correctly disconnected when command line ends.</li>
     * </ul>
     */
    @Test
    public void testRun_Command_Disconnection_00() throws Exception {

        final Command commandVersion = new Version();

        PetalsAdministration admin = runCliBlocking(
                createBasicPreferences("localhost", 7700, "petals", "petals", null),
                "-" + ShellFactory.COMMAND_SHORT_OPTION, "--", commandVersion.getName());

        assertTrue("Connection not established", systemOut.getLog().contains(SystemInfoUtils.SYSTEM_INFO_EXAMPLE));
        assertTrue("An error occurs", systemErr.getLog().isEmpty());
        assertFalse("Connection not disconnected", admin.isConnected());

    }

    /**
     * <p>
     * Test the correct disconnection when:
     * </p>
     * <ul>
     * <li>a connection is established without using the yes flag but confirming connection values,</li>
     * <li>a command requiring the connection is executed (ex: 'version' that requires a connection in the
     * implementation of the Petals Admin API mock),</li>
     * <li>the command 'disconnect' or 'exit' is not executed before to end the Petals CLI script,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>No error occurs,</li>
     * <li>The connection is established on 'connect' from preferences,</li>
     * <li>The connection is correctly disconnected.</li>
     * </ul>
     */
    @Test()
    public void testRun_Interactive_Disconnection_00() throws Exception {

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final String hostname = "localhost";

            PetalsAdminShellExtension admin = runCli(
                    createBasicPreferences("localhost", 7700, "petals", "petals", null),
                    "-h", hostname, "-n", "7700", "-u", "petals", "-p", "petals", "-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect");

            // Wait the execution of the command
            checkOut(containsString("Would you like to connect to petals:*****@" + hostname + ":7700? (y/n)"));

            // Confirmation
            systemIn.writeLine("y");

            // Wait the execution of the confirmation
            checkOut(containsString("petals-cli@" + hostname + ":7700>"));
            assertTrue("Connection not established", admin.getAdmin().isConnected());

            // Enter the command
            systemIn.writeLine("version");

            // Wait the execution of the command
            checkOut(containsString(SystemInfoUtils.SYSTEM_INFO_EXAMPLE));

            systemIn.writeLine("exit");

            waitThreadOrFail();

            assertFalse("Connection not disconnected", admin.getAdmin().isConnected());

        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Check the main entry point against an command line containing to check the host name in prompt:
     * </p>
     * <ul>
     * <li>the flag debug is disable,</li>
     * <li>connection defines on the command line with a host defined by its <b>IP address</b>,</li>
     * <li>the interactive mode,</li>
     * <li>the connection is done with 'connect' without args, and connection is confirmed,</li>
     * <li>no other command is executed.</li>
     * </ul>
     * <p>
     * as following:</p>
     * <pre>
     * {@code > ./petals-cli.sh -h 127.0.0.1 -n 7700 -u petals -p petals -C
     * petals-cli> connect
     * petals-cli@localhost:7700>
     * }</pre>
     * <p>
     * Note: the connection <b>is not a real</b> connection.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>the prompt displays connection information with an explicit hostname,</li>
     * <li>no error,</li>
     * <li>no error stacktrace is printed,</li>
     * <li>no usage is printed.</li>
     * </ul>
     */
    @Test
    public final void testRun_Interactive_ExplicitHostname_00() throws Exception {

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final String hostname = "127.0.0.1";
            final String expectedHostname = "localhost";

            runCli(createBasicPreferences("localhost", 7700, "petals", "petals", null), "-h", hostname, "-n", "7700",
                    "-u", "petals", "-p", "petals", "-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect");

            // Wait the execution of the command
            checkOut(containsString("Would you like to connect to petals:*****@" + expectedHostname + ":7700? (y/n)"));

            // Confirmation
            systemIn.writeLine("y");

            // Wait the execution of the command
            checkOut(containsString("petals-cli@" + expectedHostname + ":7700>"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Check the main entry point against an command line containing:
     * </p>
     * <ul>
     * <li>the flag debug is enabled,</li>
     * <li>the interactive mode,</li>
     * <li>the command to connect to an existing Petals node generating an error during the connection.,</li>
     * </ul>
     * <p>
     * as following:</p>
     * <pre>
     * {@code > ./petals-cli.sh -d -C
     * petals-cli> connect -h 1.1.1.1 -n 7700 -u petals -p petals
     * petals-cli>
     * }</pre>
     * <p>
     * Note: the connection <b>is not a real</b> connection, but raises an exception on the host '
     * {@link PetalsAdministrationMock#UNKNOWN_HOST}'
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>error on the command 'connect' about the connection to an unknown host,</li>
     * <li>error stack trace is printed,</li>
     * <li>no usage is printed,</li>
     * <li>the command flow is not interrupted (because entered by the user that saw the error)</li>
     * <li>the prompt is printed after error.</li>
     * </ul>
     */
    @Test
    public final void testRun_Interactive_Error_01() throws Exception {

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli(new String[] { "-d", "-C" });

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Enter the command realizing the connection
            systemIn.writeLine("connect -h " + PetalsAdministrationMock.UNKNOWN_HOST + " -n 7700 -u petals -p petals");

            // Wait the execution of the command
            Awaitility.setDefaultTimeout(Duration.TEN_SECONDS);
            checkErr(allOf(startsWith("ERROR on command '" + new Connect().getName() + "':"),
                    containsString("Caused by"), containsString("Unknown Host")));

            checkOut(containsString(Main.PROMPT_BASE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Check the main entry point against an command line containing to check the host name in prompt:
     * </p>
     * <ul>
     * <li>the flag debug is disable,</li>
     * <li>connection defines on the command line with a host defined by its <b>hostname</b>,</li>
     * <li>the interactive mode,</li>
     * <li>the connection is done with 'connect' without args, and connection is confirmed,</li>
     * <li>no other command is executed.</li>
     * </ul>
     * <p>
     * as following:</p>
     * <pre>
     * {@code > ./petals-cli.sh -h localhost -n 7700 -u petals -p petals -C
     * petals-cli> connect
     * petals-cli@localhost:7700>
     * }</pre>
     * <p>
     * Note: the connection <b>is not a real</b> connection.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>the prompt displays connection information with an explicit hostname,</li>
     * <li>no error,</li>
     * <li>no error stacktrace is printed,</li>
     * <li>no usage is printed.</li>
     * </ul>
     */
    @Test
    public final void testRun_Interactive_ExplicitHostname_01() throws Exception {

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final String hostname = "localhost";
            final String expectedHostname = "localhost";

            runCli(createBasicPreferences("localhost", 7700, "petals", "petals", null), "-h", hostname, "-n", "7700",
                    "-u", "petals", "-p", "petals", "-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect");

            // Wait the execution of the command
            checkOut(containsString("Would you like to connect to petals:*****@" + expectedHostname + ":7700? (y/n)"));

            // Confirmation
            systemIn.writeLine("y");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@" + expectedHostname + ":7700>"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Check the main entry point against an command line containing:
     * <p>
     * <ul>
     * <li>the flag debug is disable,</li>
     * <li>connection defines on the command line,</li>
     * <li>the interactive mode,</li>
     * <li>the connection is done with 'connect' without args, and connection is confirmed,</li>
     * <li>the command 'list' without arguments is executed after a 'connect'.</li>
     * </ul>
     * <p>
     * as following:</p>
     * <pre>
     * {@code > ./petals-cli.sh -h localhost -n 7700 -u petals -p petals -C
     * petals-cli> connect
     * petals-cli@localhost:7700> list
     * }</pre>
     * <p>
     * Note: the connection <b>is not a real</b> connection.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>Petals CLI is connected before entering the command,</li>
     * <li>the prompt displays connection information,</li>
     * <li>no error,</li>
     * <li>no error stacktrace is printed,</li>
     * <li>no usage is printed,</li>
     * <li>commands correctly executed.</li>
     * </ul>
     */
    @Test
    public final void testRun_Interactive_00() throws Exception {

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final String hostname = "localhost";

            runCli(createBasicPreferences("localhost", 7700, "petals", "petals", null), "-h", hostname, "-n", "7700",
                    "-u", "petals", "-p", "petals", "-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect");

            // Wait the execution of the command
            checkOut(containsString("Would you like to connect to petals:*****@" + hostname + ":7700? (y/n)"));

            // Confirmation
            systemIn.writeLine("y");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@" + hostname + ":7700>"));

            // Enter the command
            systemIn.writeLine("list");
            checkOut(containsString(Main.PROMPT_BASE + "@" + hostname + ":7700>"));

            systemIn.writeLine("exit");

            waitThreadOrFail();
        } finally {

            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }
}
