/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.connection;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.fail;

import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

import org.junit.Test;
import org.ow2.petals.cli.api.connection.AuthenticatedConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidConnectionParameterException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;

/**
 * Unit tests of {@link AuthenticatedConnectionParameters}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AuthenticatedConnectionParametersImplTest {

    private static final String HOST = "localhost";

    private static final int PORT = 12345;

    private static final String USER = "user";

    private static final String PWD = "pwd";

    private static final String PASSPHRASE = "pass-phrase";

    /**
     * Check the creation
     */
    @Test
    public void testCreation() throws UnknownHostException, InvalidConnectionParameterException {

        try {
            new AuthenticatedConnectionParametersImpl(null, PORT, USER, PWD, PASSPHRASE);
            fail("Exception '" + HostMissingException.class.getSimpleName() + "' not thrown.");
        } catch (final HostMissingException e) {
            // NOP: Expected exception
        }

        try {
            new AuthenticatedConnectionParametersImpl(HOST, 0, USER, PWD, PASSPHRASE);
            fail("Exception '" + InvalidPortException.class.getSimpleName() + "' not thrown.");
        } catch (final InvalidPortException e) {
            // NOP: Expected exception
        }

        try {
            new AuthenticatedConnectionParametersImpl(HOST, -10, USER, PWD, PASSPHRASE);
            fail("Exception '" + InvalidPortException.class.getSimpleName() + "' not thrown.");
        } catch (final InvalidPortException e) {
            // NOP: Expected exception
        }

        try {
            new AuthenticatedConnectionParametersImpl(HOST, 65536, USER, PWD, PASSPHRASE);
            fail("Exception '" + InvalidPortException.class.getSimpleName() + "' not thrown.");
        } catch (final InvalidPortException e) {
            // NOP: Expected exception
        }

        try {
            new AuthenticatedConnectionParametersImpl(HOST, PORT, null, PWD, PASSPHRASE);
            fail("Exception '" + UsernameMissingException.class.getSimpleName() + "' not thrown.");
        } catch (final UsernameMissingException e) {
            // NOP: Expected exception
        }

        try {
            new AuthenticatedConnectionParametersImpl(HOST, PORT, USER, null, PASSPHRASE);
            fail("Exception '" + PasswordMissingException.class.getSimpleName() + "' not thrown.");
        } catch (final PasswordMissingException e) {
            // NOP: Expected exception
        }

        try {
            new AuthenticatedConnectionParametersImpl(HOST, PORT, USER, PWD, null);
            new AuthenticatedConnectionParametersImpl(HOST, PORT, USER, PWD, PASSPHRASE);
        } catch (final InvalidConnectionParameterException e) {
            fail("Exception '" + InvalidConnectionParameterException.class.getSimpleName() + "' thrown.");
        }

    }

    /**
     * Check that different instances of {@link AuthenticatedConnectionParameters} can be compared
     */
    @Test
    public void testComparable() throws UnknownHostException, InvalidConnectionParameterException {

        final AuthenticatedConnectionParameters acp_1 = new AuthenticatedConnectionParametersImpl(HOST, PORT, USER,
                PWD,
                PASSPHRASE);
        assertEquals("Not equals", acp_1, new AuthenticatedConnectionParametersImpl(HOST, PORT, USER, PWD, PASSPHRASE));
        assertNotEquals("Should ot be equals because of different name", acp_1,
                new AuthenticatedConnectionParametersImpl(
                "192.168.1.1", PORT, USER, PWD, PASSPHRASE));
        assertNotEquals("Should ot be equals because of different port", acp_1,
                new AuthenticatedConnectionParametersImpl(
                HOST, 54321, USER, PWD, PASSPHRASE));
        assertNotEquals("Should ot be equals because of different user", acp_1,
                new AuthenticatedConnectionParametersImpl(
                HOST, PORT, "diff user", PWD, PASSPHRASE));
        assertNotEquals("Should ot be equals because of different pwd", acp_1,
                new AuthenticatedConnectionParametersImpl(
                HOST, PORT, USER, "diff pwd", PASSPHRASE));
        assertNotEquals("Should ot be equals because of different pass phrase", acp_1,
                new AuthenticatedConnectionParametersImpl(HOST, PORT, USER, PWD, "diff pass phrase"));
        assertNotEquals("Should ot be equals because of null pass phrase", acp_1,
                new AuthenticatedConnectionParametersImpl(HOST, PORT, USER, PWD, null));

    }

    /**
     * Check that an instance of {@link AuthenticatedConnectionParameters} can be used as a key of a {@link Map}
     */
    @Test
    public void testHashable() throws UnknownHostException, InvalidConnectionParameterException {

        final Map<AuthenticatedConnectionParameters, String> map = new HashMap<AuthenticatedConnectionParameters, String>();
        final AuthenticatedConnectionParameters acp_1 = new AuthenticatedConnectionParametersImpl(HOST, PORT, USER,
                PWD,
                PASSPHRASE);

        final String myValue = "valid-value";
        final String otherValue = "other-value";
        map.put(acp_1, myValue);
        map.put(new AuthenticatedConnectionParametersImpl("192.168.1.1", PORT, USER, PWD, PASSPHRASE), otherValue);
        map.put(new AuthenticatedConnectionParametersImpl(HOST, 54321, USER, PWD, PASSPHRASE), otherValue);
        map.put(new AuthenticatedConnectionParametersImpl(HOST, PORT, "diff user", PWD, PASSPHRASE), otherValue);
        map.put(new AuthenticatedConnectionParametersImpl(HOST, PORT, USER, "diff pwd", PASSPHRASE), otherValue);
        map.put(new AuthenticatedConnectionParametersImpl(HOST, PORT, USER, PWD, "diff pass phrase"), otherValue);

        assertEquals("Unexpected value for key", myValue, map.get(acp_1));

    }
}
