/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandInvalidArgumentException;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingArgumentException;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingOptionsException;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;

import javax.net.ssl.SSLHandshakeException;

import org.apache.mina.util.AvailablePortFinder;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.junit.rules.webserver.WebServer;

/**
 * Unit test of {@link CheckSSLHandshake}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class CheckSSLHandshakeTest extends AbstractArtifactCommandTest {

    private static final int HTTPS_PORT = AvailablePortFinder.getNextAvailable(443);

    /**
     * <p>
     * The keystore containing the public and private keys that the HTTP server will use to encrypt its communications
     * with its clients.
     * </p>
     * <p>
     * The keystore was built with following information:
     * </p>
     * <ul>
     * <li>password: srv-keystore-passwd</li>
     * <li>first name and last name: Christophe DENEUX</li>
     * <li>organization: Petals ESB</li>
     * <li>company: Linagora</li>
     * <li>town: Toulouse</li>
     * <li>state: Haute-Garonne</li>
     * <li>country code: FR</li>
     * <li>key password: srv-key-passwd</li>
     * </ul>
     * <p>
     * and generating with following command line: keytool -genkey -alias check-ssl-handshake -keyalg RSA -keystore
     * srv-keystore.jks -keysize 2048
     * </p>
     */
    private static final String SRV_KEYSTORE_RESOURCE_NAME = "org/ow2/petals/cli/shell/command/check-ssl-handshake/srv-keystore.jks";

    private static final String SRV_KEYSTORE_PASSWORD = "srv-keystore-passwd";

    private static final String SRV_KEY_PASSWORD = "srv-key-passwd";

    /**
     * <p>
     * The keystore containing the public key that the HTTP client will use to encrypt its initial communication with
     * the server.
     * </p>
     * <p>
     * The keystore was built with following commands:
     * <ol>
     * <li>keytool -export -keystore srv-keystore.jks -alias check-ssl-handshake -storepass srv-keystore-passwd -keypass
     * srv-key-passwd -file srv-public-key.cer</li>
     * <li>keytool -import -v -trustcacerts -noprompt -alias srv-check-ssl-handshake -file srv-public-key.cer -keystore
     * clt-keystore.jks -keypass clt-key-passwd -storepass clt-keystore-passwd</li>
     * </ol>
     * </p>
     */
    private static final String CLT_KEYSTORE_RESOURCE_NAME = "org/ow2/petals/cli/shell/command/check-ssl-handshake/clt-keystore.jks";

    private static final String CLT_KEYSTORE_PASSWORD = "clt-keystore-passwd";

    private static final String CLT_KEY_PASSWORD = "clt-key-passwd";

    @ClassRule
    public static final WebServer httpsServer = new WebServer(0, HTTPS_PORT, SRV_KEYSTORE_RESOURCE_NAME,
            SRV_KEYSTORE_PASSWORD, SRV_KEY_PASSWORD);

    @Rule
    public final TemporaryFolder tempFolder = new TemporaryFolder();

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final Command command = new CheckSSLHandshake();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number
     * </p>
     */
    @Test
    public void unrecognizedOption() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>the host option is provided without value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about missing hostname value
     * </p>
     */
    @Test
    public void missingHostnameValue() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.HOST_SHORT_OPTION });
            fail("CommandInvalidArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, CheckSSLHandshake.HOST_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>an unknown hostname is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about invalid hostname
     * </p>
     */
    @Test
    public void unknownHost() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        final String unknownHost = "unknown-host";
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.HOST_SHORT_OPTION, unknownHost });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, CheckSSLHandshake.HOST_SHORT_OPTION, unknownHost);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>the port option is provided without value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about missing port value
     * </p>
     */
    @Test
    public void missingPortValue() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.PORT_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, CheckSSLHandshake.PORT_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>an invalid port (not a number) is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about invalid port
     * </p>
     */
    @Test
    public void invalidPort() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        final String invalidPort = "invalid-port-value";
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.PORT_SHORT_OPTION, invalidPort });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, CheckSSLHandshake.PORT_SHORT_OPTION, invalidPort);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>an invalid port (value out of limits) is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about invalid port
     * </p>
     */
    @Test
    public void tooBigPort() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        final String invalidPort = "99999";
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.PORT_SHORT_OPTION, invalidPort });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, CheckSSLHandshake.PORT_SHORT_OPTION, invalidPort);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>the keystore file option is provided without value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about missing keystore file value
     * </p>
     */
    @Test
    public void missingKeystoreFileValue() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>an inexisting keystore file is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about invalid hostname
     * </p>
     */
    @Test
    public void invalidKeystoreFile() throws Exception {

        final File keystoreFolder = this.tempFolder.newFolder();
        final File inexistingKeystoreFile = new File(keystoreFolder, "inexisting-keystore-file");

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION,
                    inexistingKeystoreFile.getAbsolutePath(), "-" + CheckSSLHandshake.KEYSTORE_PASS_SHORT_OPTION,
                    "passphrase" });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION,
                    inexistingKeystoreFile.getAbsolutePath());
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>the keystore passphrase option is provided without value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about missing keystore passphrase value
     * </p>
     */
    @Test
    public void missingKeystorePassValue() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.KEYSTORE_PASS_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, CheckSSLHandshake.KEYSTORE_PASS_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>a keystore passphrase is provided without keystore file</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about missing keystore file option
     * </p>
     */
    @Test
    public void keystorePassWithoutKeystoreFile() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.KEYSTORE_PASS_SHORT_OPTION, "passphrase" });
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>a keystore file is provided without keystore passphrase</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about missing keystore passphrase option
     * </p>
     */
    @Test
    public void keystoreFileWithoutKeystorePassPhrase() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION, "keystore-file" });
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, CheckSSLHandshake.KEYSTORE_PASS_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" about a succeeded SSL handshake check
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>SSL handshake succeeds,</li>
     * <li>no connection to a Petals ESB container is require.</li>
     * </ul>
     */
    @Test
    public void succeeded() throws CommandException, URISyntaxException {

        final URL cltKeystoreUrl = Thread.currentThread().getContextClassLoader()
                .getResource(CLT_KEYSTORE_RESOURCE_NAME);
        assertNotNull(cltKeystoreUrl);

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new CheckSSLHandshake();
        command.setShell(shell);

        command.execute(new String[] { "-" + CheckSSLHandshake.HOST_SHORT_OPTION, "localhost",
                "-" + CheckSSLHandshake.PORT_SHORT_OPTION, String.valueOf(HTTPS_PORT),
                "-" + CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION, new File(cltKeystoreUrl.toURI()).getAbsolutePath(),
                "-" + CheckSSLHandshake.KEYSTORE_PASS_SHORT_OPTION, CLT_KEYSTORE_PASSWORD });

        assertTrue(shell.getOutput().contains(CheckSSLHandshake.CHECK_OK_MESSAGE));
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" about a failed SSL handshake check
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about the failed SSL handshake,</li>
     * <li>no connection to a Petals ESB container is require.</li>
     * </ul>
     */
    @Test
    public void failed() throws CommandException, URISyntaxException {

        final URL cltKeystoreUrl = Thread.currentThread().getContextClassLoader()
                .getResource(CLT_KEYSTORE_RESOURCE_NAME);
        assertNotNull(cltKeystoreUrl);

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new CheckSSLHandshake();
        command.setShell(shell);

        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.HOST_SHORT_OPTION, "localhost",
                    "-" + CheckSSLHandshake.PORT_SHORT_OPTION, String.valueOf(HTTPS_PORT) });
            fail("CommandException is not thrown");
        } catch (final CommandException e) {
            assertTrue(e.getCause() instanceof SSLHandshakeException);
        }
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" about a succeeded SSL handshake check trusting all certificates
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>SSL handshake succeeds,</li>
     * <li>no connection to a Petals ESB container is require.</li>
     * </ul>
     */
    @Test
    public void succeededTrustingAllCertificates() throws CommandException, URISyntaxException {

        final URL cltKeystoreUrl = Thread.currentThread().getContextClassLoader()
                .getResource(CLT_KEYSTORE_RESOURCE_NAME);
        assertNotNull(cltKeystoreUrl);

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new CheckSSLHandshake();
        command.setShell(shell);

        command.execute(new String[] {
                "-" + CheckSSLHandshake.PORT_SHORT_OPTION, String.valueOf(HTTPS_PORT),
                "--" + CheckSSLHandshake.TRUST_ALL_CERTS_LONG_OPTION });

        assertTrue(shell.getOutput().contains(CheckSSLHandshake.CHECK_OK_MESSAGE));
    }

    /**
     * <p>
     * Test of the command "check-ssl-handshake" against argument management, where:
     * </p>
     * <ul>
     * <li>a keystore file option is provided,</li>
     * <li>and the trust all certificates option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about option incompatibility
     * </p>
     */
    @Test
    public void keystoreFileAndTrustAllCerts() throws CommandException {

        final Command command = new CheckSSLHandshake();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION, "keystore.jks",
                    "-" + CheckSSLHandshake.KEYSTORE_PASS_SHORT_OPTION, CLT_KEYSTORE_PASSWORD,
                    "--" + CheckSSLHandshake.TRUST_ALL_CERTS_LONG_OPTION });
            fail("CommandTooManyArgumentsException is not thrown");
        } catch (final CommandTooManyArgumentsException e) {
            final String errorMsg = e.getMessage();
            assertTrue("Error label is missing.",
                    errorMsg.startsWith(CommandTooManyArgumentsException.TOO_MANY_ARGUMENTS));
            assertTrue("Not expected incompatible options",
                    errorMsg.contains(CheckSSLHandshake.TRUST_ALL_CERTS_LONG_OPTION)
                            && errorMsg.contains(CheckSSLHandshake.KEYSTORE_FILE_SHORT_OPTION));
        }
    }

}
