/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandInvalidArgumentException;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingOptionsException;

import org.junit.Test;
import org.ow2.petals.admin.junit.ContainerAdministrationMock;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * Unit test of {@link LoggerSet}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class LoggerSetTest extends AbstractConnectedShellTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final LoggerSet command = new LoggerSet();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "logger-set" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name is missing,</li>
     * <li>the logger level is missing.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing option.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws CommandException {

        final LoggerSet command = new LoggerSet();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] {});
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, LoggerSet.LOGGER_NAME_SHORT_OPTION, LoggerSet.LEVEL_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "logger-set" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name is provided,</li>
     * <li>the logger level is missing.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing option.
     * </p>
     */
    @Test
    public void testArgumentsError_1() throws CommandException {

        final LoggerSet command = new LoggerSet();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + LoggerSet.LOGGER_NAME_SHORT_OPTION, "mylogger" });
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, LoggerSet.LEVEL_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "logger-set" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name is missing,</li>
     * <li>the logger level is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing option.
     * </p>
     */
    @Test
    public void testArgumentsError_2() throws CommandException {

        final LoggerSet command = new LoggerSet();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + LoggerSet.LEVEL_SHORT_OPTION, "INFO" });
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, LoggerSet.LOGGER_NAME_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "logger-set" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name is provided without argument,</li>
     * <li>the logger level is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_3() throws CommandException {

        final LoggerSet command = new LoggerSet();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + LoggerSet.LOGGER_NAME_SHORT_OPTION,
                    "-" + LoggerSet.LEVEL_SHORT_OPTION, "INFO" });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue("The option '-" + LoggerSet.LOGGER_NAME_SHORT_OPTION
                    + "' has a missing argument.",
                    LoggerSet.LOGGER_NAME_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(LoggerSet.LOGGER_NAME_SHORT_OPTION));
        }
    }

    /**
     * <p>
     * Test of the command "logger-set" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name is provided,</li>
     * <li>the logger level is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_4() throws CommandException {

        final LoggerSet command = new LoggerSet();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + LoggerSet.LOGGER_NAME_SHORT_OPTION, "mylogger",
                    "-" + LoggerSet.LEVEL_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue(
                    "The option '-" + LoggerSet.LEVEL_SHORT_OPTION + "' has a missing argument.",
                    LoggerSet.LEVEL_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(LoggerSet.LEVEL_SHORT_OPTION));
        }
    }

    /**
     * <p>
     * Test of the command "logger-set" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name is provided,</li>
     * <li>the logger level is provided with an invalid value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the invalid value.
     * </p>
     */
    @Test
    public void testArgumentsError_5() throws CommandException {

        final LoggerSet command = new LoggerSet();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + LoggerSet.LOGGER_NAME_SHORT_OPTION, "mylogger",
                    "-" + LoggerSet.LEVEL_SHORT_OPTION,
                    ContainerAdministrationMock.INVALID_LOG_LEVEL });
            fail("CommandInvalidArgumentException is not thrown through CommandException.");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, LoggerSet.LEVEL_SHORT_OPTION,
                    ContainerAdministrationMock.INVALID_LOG_LEVEL);
        }
    }

    /**
     * <p>
     * Test of the command "logger-set" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name is provided but does not exist on the container side,
     * </li>
     * <li>the logger level is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the inexisting logger.
     * </p>
     */
    @Test
    public void testArgumentsError_6() throws CommandException {

        final LoggerSet command = new LoggerSet();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + LoggerSet.LOGGER_NAME_SHORT_OPTION,
                    ContainerAdministrationMock.INVALID_LOGGER_NAME,
                    "-" + LoggerSet.LEVEL_SHORT_OPTION, "INFO" });
            fail("CommandInvalidArgumentException is not thrown through CommandException.");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, LoggerSet.LOGGER_NAME_SHORT_OPTION,
                    ContainerAdministrationMock.INVALID_LOGGER_NAME);
        }
    }

    /**
     * <p>
     * Test of the command "logger-set" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_7() throws CommandException {

        final LoggerSet command = new LoggerSet();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "logger-set" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new LoggerSet();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
