/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
import org.ow2.petals.admin.junit.ContainerAdministrationMock;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * Unit test of {@link Loggers}
 *
 * @author Christophe DENEUX - Linagora
 *
 */
public class LoggersTest extends AbstractConnectedShellTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final Loggers command = new Loggers();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "loggers" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name regexp is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws CommandException {

        final Loggers command = new Loggers();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + Loggers.LOGGER_NAME_REGEXP_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue(
                    "The option '-" + Loggers.LOGGER_NAME_REGEXP_SHORT_OPTION + "' has a missing argument.",
                    Loggers.LOGGER_NAME_REGEXP_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(Loggers.LOGGER_NAME_REGEXP_SHORT_OPTION));
        }
    }

    /**
     * <p>
     * Test of the command "loggers" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name regexp is not provided.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>loggers are listed.</li>
     * </ul>
     */
    @Test
    public void testArgumentsError_1() throws CommandException {

        final Loggers command = new Loggers();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] {});
        assertFalse("No logger is displayed !", this.dummyShellWrapper.getOut().toString().isEmpty());
    }

    /**
     * <p>
     * Test of the command "loggers" against argument management, where:
     * </p>
     * <ul>
     * <li>the logger name regexp is provided without its option.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_2() throws CommandException {

        final Loggers command = new Loggers();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { ContainerAdministrationMock.LOGGER1_LOGGER_NAME });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "loggers" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_3() throws CommandException {

        final Loggers command = new Loggers();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "loggers" against display, where:
     * </p>
     * <ul>
     * <li>the a logger name regexp matching only one logger is provided.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>one logger is listed with its level.</li>
     * </ul>
     */
    @Test
    public void testExecute_0() throws CommandException {

        final Loggers command = new Loggers();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + Loggers.LOGGER_NAME_REGEXP_SHORT_OPTION,
                ContainerAdministrationMock.LOGGER1_LOGGER_NAME });
        assertEquals("The displayed information are incorrect!", this.dummyShellWrapper.getOut().toString(),
                ContainerAdministrationMock.LOGGER1_LOGGER_NAME + " " + ContainerAdministrationMock.LOGGER1_LEVEL
                        + System.getProperty("line.separator"));
    }

    /**
     * <p>
     * Test of the command "loggers" against display of the root logger, where:
     * </p>
     * <ul>
     * <li>no regexp is provided.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the root logger is displayed as '<code>&lt;root&gt;</code>'.</li>
     * </ul>
     */
    @Test
    public void testExecute_1() throws CommandException {

        final Loggers command = new Loggers();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] {});
        assertTrue(
                "The displayed information are incorrect !",
                this.dummyShellWrapper
                        .getOut()
                        .toString()
                .startsWith("<root> " + ContainerAdministrationMock.ROOT_LEVEL + System.getProperty("line.separator")));
    }

    /**
     * <p>
     * Test of the command "loggers" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new Loggers();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
