/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingOptionsException;

import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandMissingYesFlagException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * Unit test of {@link MoveContainer}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MoveContainerTest extends AbstractConnectedShellTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final Command command = new MoveContainer();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where:
     * </p>
     * <ul>
     * <li>only the option '--target-host' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of options.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_01() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_HOST_LONG_OPTION, "dummy" });
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where:
     * </p>
     * <ul>
     * <li>only the option '--target-host' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of options.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_02() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_HOST_LONG_OPTION, "dummy" });
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where:
     * </p>
     * <ul>
     * <li>only the option '--target-port' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of options.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_03() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_PORT_LONG_OPTION, "dummy" });
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where:
     * </p>
     * <ul>
     * <li>only the option '--target-user' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of options.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_04() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_USER_LONG_OPTION, "dummy" });
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where:
     * </p>
     * <ul>
     * <li>only the option '--target-password' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of options.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_05() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_PWD_LONG_OPTION, "dummy" });
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where:
     * </p>
     * <ul>
     * <li>only the option '--target-passphrase' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of options.
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testArgumentsError_06() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_PASSPHRASE_LONG_OPTION, "dummy" });
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where the options required for an attachment
     * are provided:
     * </p>
     * <ul>
     * <li>the option '--target-domain' is given with an argument</li>
     * <li>the option '--target-pass-phrase' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing yes flag.
     * </p>
     */
    @Test(expected = CommandMissingYesFlagException.class)
    public void testArgumentsError_07() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_DOMAIN_LONG_OPTION, "dummyDomain",
                "--" + MoveContainer.TARGET_PASSPHRASE_LONG_OPTION, "dummyPassPhrase" });
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where the options required for an attachment
     * are provided:
     * </p>
     * <ul>
     * <li>the option '--target-domain' is given with an argument</li>
     * <li>the option '--target-port' is given with an invalid value argument (not a number)</li>
     * <li>the option '-y' as yes flag</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the value of the target port.
     * </p>
     */
    @Test(expected = CommandUnparsableArgumentException.class)
    public void testArgumentsError_08() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        final String invalidValue = "dummyPort";
        try {
            command.execute(new String[] { "--" + MoveContainer.TARGET_DOMAIN_LONG_OPTION, "dummyDomain",
                    "--" + MoveContainer.TARGET_PORT_LONG_OPTION, invalidValue,
                    "-" + MoveContainer.YESFLAG_SHORT_OPTION });
        } catch (final CommandUnparsableArgumentException e) {
            assertEquals("Unexpected option", MoveContainer.TARGET_PORT_LONG_OPTION, e.getOption().getLongOpt());
            assertEquals("Unexpected invalid value", invalidValue, e.getUnparsableValue());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where:
     * </p>
     * <ul>
     * <li>the option '--target-domain' is given with an argument</li>
     * <li>the option '--target-pass-phrase' is missing</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing option '--target-pass-phrase'
     * </p>
     */
    @Test(expected = CommandMissingOptionsException.class)
    public void testArgumentsError_09() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        try {
            command.execute(new String[] { "--" + MoveContainer.TARGET_DOMAIN_LONG_OPTION, "dummyDomain" });
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, MoveContainer.TARGET_PASSPHRASE_LONG_OPTION);
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "move-container" against argument management, where the options required for a detachment are
     * provided:
     * </p>
     * <ul>
     * <li>the option '-y' is missing</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing yes flag.
     * </p>
     */
    @Test(expected = CommandMissingYesFlagException.class)
    public void testDetach_ArgumentsError_00() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] {});
    }

    /**
     * <p>
     * Test of the command "move-container" where all arguments are correctly set for an attachment:
     * </p>
     * <ul>
     * <li>the option '--target-domain' is given with an argument</li>
     * <li>the option '--target-pass-phrase' is given with an argument</li>
     * <li>the option '-y' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void testAttach_00() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_DOMAIN_LONG_OPTION, "dummyDomain",
                "--" + MoveContainer.TARGET_PASSPHRASE_LONG_OPTION, "dummyPassPhrase",
                "-" + MoveContainer.YESFLAG_SHORT_OPTION });
    }

    /**
     * <p>
     * Test of the command "move-container" where all arguments are correctly set for an attachment:
     * </p>
     * <ul>
     * <li>the option '--target-domain' is given with an argument</li>
     * <li>the option '--target-pass-phrase' is given with an argument</li>
     * <li>the option '--target-port' is set to a valid value</li>
     * <li>the option '--target-host' is set to a valid value</li>
     * <li>the option '--target-user' is set to a valid value</li>
     * <li>the option '--target-pwd' is set to a valid value</li>
     * <li>the option '-y' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void testAttach_0() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + MoveContainer.TARGET_DOMAIN_LONG_OPTION, "dummyDomain",
                "--" + MoveContainer.TARGET_PASSPHRASE_LONG_OPTION, "dummyPassPhrase",
                "--" + MoveContainer.TARGET_PORT_LONG_OPTION, "12345", "--" + MoveContainer.TARGET_HOST_LONG_OPTION,
                "localhost", "--" + MoveContainer.TARGET_USER_LONG_OPTION, "dummyUser",
                "--" + MoveContainer.TARGET_PWD_LONG_OPTION, "dummyPwd", "-" + MoveContainer.YESFLAG_SHORT_OPTION });
    }

    /**
     * <p>
     * Test of the command "move-container" where all arguments are correctly set for a detachment:
     * </p>
     * <ul>
     * <li>the option '-y' is given with an argument</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void testDetach_00() throws CommandException {

        final Command command = new MoveContainer();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "-" + MoveContainer.YESFLAG_SHORT_OPTION });
    }

    /**
     * <p>
     * Test of the command "move-container" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new MoveContainer();
        command.setShell(shell);

        command.execute(new String[] {});
    }

}
