/**
 * Copyright (c) 2015-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;
import static org.ow2.petals.cli.shell.command.ArtifactBasedCommand.ARTIFACT_SHORT_OPTION;

import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * Unit test of {@link ReloadConfiguration}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ReloadConfigurationTest extends AbstractArtifactCommandTest {

    @Before
    public void deployAndStartComponent() throws Exception {
        super.deployStartArtifact(COMPONENT_URL);
    }

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final Command command = new ReloadConfiguration();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "reload-configuration" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws Exception {

        final Command command = new ReloadConfiguration();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "reload-configuration" against argument management, where:
     * </p>
     * <ul>
     * <li>several URLs are given</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_SeveralUrlsGiven() throws Exception {

        final Command command = new ReloadConfiguration();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + URL_SHORT_OPTION, COMPONENT_URL.toString(), SA_URL.toString() });

            fail("Exception 'CommandTooManyArgumentsException' is not thrown");
        } catch (final CommandTooManyArgumentsException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandTooManyArgumentsException.TOO_MANY_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "reload-configuration", where:
     * </p>
     * <ul>
     * <li>the component is given through its URL</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the configuration is correctly reloaded.
     * </p>
     */
    @Test
    public void reloadConfiguration_OneArtifactByURL() throws Exception {

        // Check test env
        final ArtifactAdministration admin = api.getSingleton().newArtifactAdministration();
        final List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        final Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());

        // Execute test
        final Command command = new ReloadConfiguration();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + URL_SHORT_OPTION, COMPONENT_URL.toString() });

        // Check test result
        // TODO: Add something to check the execution of the reload
    }

    /**
     * <p>
     * Test of the command "reload-configuration" against argument management, where:
     * </p>
     * <ul>
     * <li>several artifact name are given</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void reloadConfiguration_SeveralArtifactName() throws Exception {

        // Execute test
        final Command command = new ReloadConfiguration();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, COMPONENT_ID, "2nd-component" });
            fail("Exception 'CommandTooManyArgumentsException' is not thrown");
        } catch (final CommandTooManyArgumentsException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandTooManyArgumentsException.TOO_MANY_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "reload-configuration", where:
     * </p>
     * <ul>
     * <li>the component is given through its name</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the configuration is correctly reloaded.
     * </p>
     */
    @Test
    public void reloadConfiguration_ByArtifactName() throws Exception {

        // Check test env
        final ArtifactAdministration admin = api.getSingleton().newArtifactAdministration();
        final List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        final Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());

        // Execute test
        final Command command = new ReloadConfiguration();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, COMPONENT_ID });

        // Check test result
        // TODO: Add something to check the execution of the reload
    }

    /**
     * <p>
     * Test of the command "reload-configuration" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new ReloadConfiguration();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
