/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;

import java.net.URL;
import java.util.List;

import org.junit.Test;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactTypeIsNeededException;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class StartArtifactTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final StartArtifact command = new StartArtifact();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "start-artifact" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws Exception {

        final StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    @Test
    public void testStartOneArtifactByFileURL() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();

        admin.deployAndStartArtifact(COMPONENT_URL, true);

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        Artifact artifact = artifacts.get(0);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + URL_SHORT_OPTION, COMPONENT_URL.toString() });

        artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());
    }

    @Test
    public void testStartOneArtifactByMavenURL() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();

        final URL componentURL = new URL("mvn:org.ow2.petals/petals-bc-soap/4.4.2/zip");
        admin.deployAndStartArtifact(componentURL, true);

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        Artifact artifact = artifacts.get(0);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + URL_SHORT_OPTION, componentURL.toString() });

        artifact = admin.getArtifactInfo(componentURL);
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by artifact type and artifact
     * identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly started</li>
     * </ul>
     */
    @Test
    public void testStartOneArtifactByTypeAndName_0() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, true);

        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();
        String type = artifact.getType();

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if component is deployed", 1, artifacts.size());
        artifact = artifacts.get(0);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name,
                "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, type });

        artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by artifact type and artifact
     * identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly started</li>
     * </ul>
     */
    @Test
    public void testStartOneArtifactByTypeAndName_1() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, true);

        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if component is deployed", 1, artifacts.size());
        artifact = artifacts.get(0);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name });

        artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by its type and its identifier
     * where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing artifact type</li>
     * <li>the artifacts are not started</li>
     * </ul>
     */
    @Test
    public void testStartOneArtifactByTypeAndName_2() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, true);

        admin.deployAndStartArtifact(SA_SOAP_ID_URL, true);
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if components are deployed", 10, artifacts.size());

        final Artifact artifactComp = admin.getArtifactInfo(COMPONENT_URL);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifactComp).getState() != ArtifactState.State.STARTED);

        final Artifact artifactSa = admin.getArtifactInfo(SA_SOAP_ID_URL);
        assertTrue("Check if sa is stopped", ((ArtifactState) artifactSa).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name });
            fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactTypeIsNeededException",
                    e.getCause() instanceof ArtifactTypeIsNeededException);
        }

        assertTrue("Check if component is stopped",
                ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState() != ArtifactState.State.STARTED);
        assertTrue("Check if sa is stopped",
                ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState() != ArtifactState.State.STARTED);
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by its type and its identifier
     * where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>the right artifact is started</li>
     * <li>the other artifact is not started</li>
     * </ul>
     */
    @Test
    public void testStartOneArtifactByTypeAndName_3() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, true);

        admin.deployAndStartArtifact(SA_SOAP_ID_URL, true);
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();
        final String type = artifact.getType();

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if components are deployed", 10, artifacts.size());

        final Artifact artifactComp = admin.getArtifactInfo(COMPONENT_URL);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifactComp).getState() != ArtifactState.State.STARTED);

        final Artifact artifactSa = admin.getArtifactInfo(SA_SOAP_ID_URL);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifactSa).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name,
                "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, type });

        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        assertTrue("Check if sa is stopped",
                ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState() != ArtifactState.State.STARTED);
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by its type and its identifier
     * where:
     * </p>
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about the bad number argument</li>
     * <li>the artifacts are always stopped</li>
     * </ul>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testStartOneArtifactByTypeAndName_4() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, true);

        admin.deployAndStartArtifact(SA_SOAP_ID_URL, true);
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        final String type = artifact.getType();

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if components are deployed", 10, artifacts.size());

        final Artifact artifactComp = admin.getArtifactInfo(COMPONENT_URL);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifactComp).getState() != ArtifactState.State.STARTED);

        final Artifact artifactSa = admin.getArtifactInfo(SA_SOAP_ID_URL);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifactSa).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, type });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Check if component is stopped",
                    ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState() != ArtifactState.State.STARTED);
            assertTrue("Check if sa is stopped",
                    ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState() != ArtifactState.State.STARTED);
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by its type and its identifier
     * where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifacts are always stopped</li>
     * </ul>
     */
    @Test
    public void testStartOneArtifactByTypeAndName_5() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, true);

        admin.deployAndStartArtifact(SA_SOAP_ID_URL, true);
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        final String identifier = artifact.getName();

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if components are deployed", 10, artifacts.size());

        final Artifact artifactComp = admin.getArtifactInfo(COMPONENT_URL);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifactComp).getState() != ArtifactState.State.STARTED);

        final Artifact artifactSa = admin.getArtifactInfo(SA_SOAP_ID_URL);
        assertTrue("Check if sa is stopped", ((ArtifactState) artifactSa).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            assertFalse("The artifact is a SE", "SE".equals(artifact.getType()));
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, identifier,
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, "SE" });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactNotFoundException",
                    e.getCause() instanceof ArtifactNotFoundException);
        }

        assertTrue("Check if component is always stopped",
                ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState() != ArtifactState.State.STARTED);
        assertTrue("Check if sa is always stopped",
                ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState() != ArtifactState.State.STARTED);
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by artifact type and artifact
     * identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifact is always stopped</li>
     * </ul>
     */
    @Test
    public void testStartOneArtifactByTypeAndName_6() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, true);

        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String identifier = artifact.getName();

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if component is deployed", 1, artifacts.size());
        artifact = artifacts.get(0);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            assertFalse("The artifact is a SE", "SE".equals(artifact.getType()));
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, identifier,
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, "SE" });
            fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactNotFoundException",
                    e.getCause() instanceof ArtifactNotFoundException);
        }

        artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by artifact type and artifact
     * identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifact is always stopped</li>
     * </ul>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testStartOneArtifactByTypeAndName_7() throws Exception {

        ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, true);

        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String type = artifact.getType();

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if component is deployed", 1, artifacts.size());
        artifact = artifacts.get(0);
        assertTrue("Check if component is stopped",
                ((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, type });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandBadArgumentNumberException e) {
            // Expected exception
            artifact = admin.getArtifactInfo(COMPONENT_URL);
            assertTrue("Check if component is stopped",
                    ((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "start-artifact" to start a not started artifact identified by artifact type and artifact
     * identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a artifact type option is provided without value,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about artifact not found
     * </p>
     */
    @Test
    public void testStartOneArtifactByTypeAndName_8() throws Exception {
        this.deployStartArtifact(COMPONENT_URL);

        Component a = (Component) ArtifactUtils.createArtifact(COMPONENT_URL);

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        String identifier = a.getName();
        try {
            assertFalse("The artifact is a SE", "SE".equals(a.getType()));
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, identifier,
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue("The option '-" + StartArtifact.TYPE_SHORT_OPTION + "' has a missing argument.",
                    StartArtifact.TYPE_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(StartArtifact.TYPE_SHORT_OPTION));
        }
    }

    /**
     * Test of the command "start-artifact" against too many argument on option '-u'.
     */
    @Test
    public void testStartManyArtifactThrownException_0() throws Exception {

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + URL_SHORT_OPTION, COMPONENT_URL.toString(), SA_URL.toString() });
        } catch (CommandException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandTooManyArgumentsException.TOO_MANY_ARGUMENTS));
        }
    }

    /**
     * Test of the command "start-artifact" against missing argument on option '-u'.
     */
    @Test
    public void testStartManyArtifactThrownException_1() throws Exception {

        StartArtifact command = new StartArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + URL_SHORT_OPTION });
        } catch (CommandException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
        }
    }

    /**
     * <p>
     * Test of the command "start-artifact" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new StartArtifact();
        command.setShell(shell);

        command.execute(new String[] {});
    }

}
