/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.endsWith;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.net.UnknownHostException;

import org.junit.Test;
import org.ow2.petals.cli.Main;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.ConnectionCommand;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingYesFlagException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.InvalidConnectionParameterException;
import org.ow2.petals.cli.api.shell.ShellExtension;
import org.ow2.petals.cli.base.junit.shell.PetalsInteractiveCliUtils;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.shell.PetalsCli;
import org.ow2.petals.cli.shell.ShellFactory;
import org.ow2.petals.cli.shell.extension.PetalsAdminShellExtension;
/**
 * Unit test of {@link Stop}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class StopTest extends AbstractConnectedShellTest {

    /**
     * The shell {@link PetalsCli} establishes automatically a connection. Here,
     * the default connection parameters.
     */
    private final ConnectionParameters defaultConnectionParameters;
    
    private final static String EXPECTED_CMD_USAGE = "usage: stop [-s] [-y]"
            + System.getProperty("line.separator");

    public StopTest() {
        try {
            this.defaultConnectionParameters = new AuthenticatedConnectionParametersImpl("localhost",
                    7700, "petals", "petals", null);
        } catch (final UnknownHostException e) {
            // NOP, the exception can not occur
            throw new RuntimeException("An unexpected exception occurs", e);
        } catch (final InvalidConnectionParameterException e) {
            // NOP, the exception can not occur
            throw new RuntimeException("An unexpected exception occurs", e);
        }
    }

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final Stop command = new Stop();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertEquals("Unexpected command usage", EXPECTED_CMD_USAGE, usage);
    }

    /**
     * <p>
     * Test of the command "stop" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_00() throws CommandException {

        final Stop command = new Stop();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "stop" against argument management, where:
     * </p>
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with connection parametern</li>
     * <li>without yes flag,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing yes flag.
     * </p>
     */
    @Test
    public void testArgumentsError_01() throws Exception {

        final Stop command = new Stop();
        final PetalsCli shell = new PetalsCli(new String[] { command.getName(), "-" + Stop.SHUTDOWN_SHORT_OPTION },
                false, false, null, new ShellExtension[] { new PetalsAdminShellExtension() }, null);
        shell.registersCommand(command);
        final ConnectionCommand connect = new Connect();
        connect.setShell(shell);
        shell.registersConnectionCommand(connect);
        shell.setConnectionParameters(this.defaultConnectionParameters);
        command.setShell(shell);

        shell.run();

        assertTrue("It's not the expected error message.",
                systemErr.getLog().contains(CommandMissingYesFlagException.MISSING_YESFLAG));

    }

    /**
     * <p>
     * Test of the command "stop" against argument management, where:
     * </p>
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with the yes flag at command line level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * <p>
     * Expected results: No error occurs about missing yes flag
     * </p>
     */
    @Test
    public void testArguments_00() {

        runCliBlocking("-" + Connect.HOST_SHORT_OPTION, "localhost",
                "-" + Connect.PORT_SHORT_OPTION, "7700", "-" + Connect.USER_SHORT_OPTION, "petals",
                "-" + Connect.PWD_SHORT_OPTION, "petals", "-" + ShellFactory.YESFLAG_SHORT_OPTION,
                "-" + ShellFactory.COMMAND_SHORT_OPTION, "--", new Stop().getName(), "-" + Stop.SHUTDOWN_SHORT_OPTION);

        assertTrue("An error message is print", systemErr.getLog().isEmpty());

    }

    /**
     * <p>
     * Test of the command "stop" against argument management, where:
     * </p>
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * <p>
     * Expected results: No error occurs about missing yes flag
     * </p>
     */
    @Test
    public void testArguments_01() {

        runCliBlocking("-" + Connect.HOST_SHORT_OPTION, "localhost",
                "-" + Connect.PORT_SHORT_OPTION, "7700", "-" + Connect.USER_SHORT_OPTION, "petals",
                "-" + Connect.PWD_SHORT_OPTION, "petals", "-" + ShellFactory.COMMAND_SHORT_OPTION, "--",
                new Stop().getName(), "-" + Stop.SHUTDOWN_SHORT_OPTION, "-" + Stop.YESFLAG_SHORT_OPTION);

        assertTrue("An error message is print", systemErr.getLog().isEmpty());

    }

    /**
     * <p>
     * Test of the command "stop" against argument management, where:
     * </p>
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>a confirmation message is displayed,</li>
     * <li>after confirmation, the command is executed displaying the prompt</li>
     * </ul>
     */
    @Test
    public void testArgumentsError_02() throws Exception {

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli("-C");
    
            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect -h localhost -n 7700 -u petals -p petals");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            // Enter the command
            systemIn.writeLine(new Stop().getName() + " -s");

            // Wait the execution of the command
            checkOut(endsWith(Stop.SHUTDOWN_CONFIRMATION_MSG));

            // Enter confirmation
            systemIn.writeLine("y");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();
        }
        finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Test of the command "stop" against argument management, where:
     * </p>
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with the yes flag at command line level,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no confirmation message is displayed,</li>
     * <li>the command is executed displaying the prompt</li>
     * </ul>
     */
    @Test
    public void testArguments_02() throws Exception {

        try {

            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli("-" + ShellFactory.YESFLAG_SHORT_OPTION, "-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect -h localhost -n 7700 -u petals -p petals");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            // Enter the command
            systemIn.writeLine(new Stop().getName() + " -s");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Test of the command "stop" against argument management, where:
     * </p>
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no confirmation message is displayed,</li>
     * <li>the command is executed displaying the prompt</li>
     * </ul>
     */
    @Test
    public void testArguments_03() throws Exception {

        try {

            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli("-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect -h localhost -n 7700 -u petals -p petals");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            // Enter the command
            systemIn.writeLine(new Stop().getName() + " -s -y");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Test of the command "stop" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new Stop();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
