/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.HashMap;
import java.util.Map;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.junit.PetalsAdministrationApi;
import org.ow2.petals.admin.junit.utils.ContainerUtils;
import org.ow2.petals.admin.junit.utils.DomainUtils;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.PortType;
import org.ow2.petals.admin.topology.Domain;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.junit.DefaultAlias;
import org.ow2.petals.cli.junit.TemporaryPreferencesFile;
import org.ow2.petals.cli.pref.PreferencesImpl;
import org.ow2.petals.cli.shell.extension.PetalsAdminShellExtension;

/**
 * Unit test of {@link TopologyList}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class TopologyListTest {

    @Rule
    public final TemporaryPreferencesFile tempPrefFile = new TemporaryPreferencesFile();

    @Rule
    public final PetalsAdministrationApi adminApi = new PetalsAdministrationApi();

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final Command command = new TopologyList();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }
    
    /**
     * <p>
     * Test of the command "topology-list" against argument management, where:
     * </p>
     * <ul>
     * <li>the pass phrase is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void passphraseOptionWithoutArg() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        shell.onConnectionEstablished();
        final Command command = new TopologyList();
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + TopologyList.PASSPHRASE_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue("The option '-" + TopologyList.PASSPHRASE_SHORT_OPTION + "' has a missing argument.",
                    TopologyList.PASSPHRASE_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(TopologyList.PASSPHRASE_SHORT_OPTION));
        }
    }

    /**
     * <p>
     * Test of the command "topology-list" against argument management, where:
     * </p>
     * <ul>
     * <li>no passphrase is defined at default connection level,</li>
     * <li>the passphrase is not provided,</li>
     * <li>default connection used,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no critical topology information are printed.</li>
     * </ul>
     */
    @Test
    public void noPassPhraseGivenWithDefaultConnectionNotIncludingPassPhrase() throws Exception {

        // Set the test environment
        this.tempPrefFile.addAlias(new DefaultAlias("my-alias").setHostName(ContainerUtils.CONTAINER_HOST)
                .setPort(Integer.toString(ContainerUtils.CONTAINER_JMX_PORT)).setJmxUser(ContainerUtils.CONTAINER_USER)
                .setJmxPwd(ContainerUtils.CONTAINER_PWD));
        this.adminApi.registerDomain().registerContainer();
        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final StringStreamShell shell = new StringStreamShell(pref);
        shell.addExtension(new PetalsAdminShellExtension());
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.execute(new String[] { "-" + Connect.YESFLAG_SHORT_OPTION });

        // Execute
        final Command command = new TopologyList();
        command.setShell(shell);
        command.execute(new String[] {});

        // Assertions
        final String output = shell.getOutput();
        assertFalse("No topology information is displayed !", output.isEmpty());
        assertTrue("JMX username not masked", shell.getOutput().contains("** JMX username: ********"));
        assertTrue("JMX password not masked", shell.getOutput().contains("** JMX password: ********"));
    }

    /**
     * <p>
     * Test of the command "topology-list" against argument management, where:
     * </p>
     * <ul>
     * <li>a correct passphrase is defined at default connection level,</li>
     * <li>no passphrase is provided on command line,</li>
     * <li>default connection used,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>critical topology information are printed, because the right passphrase is set at alias level.</li>
     * </ul>
     */
    @Test
    public void noPassPhraseGivenWithDefaultConnectionIncludingPassPhrase() throws Exception {

        // Set the test environment
        this.tempPrefFile.addAlias(new DefaultAlias("my-alias").setHostName(ContainerUtils.CONTAINER_HOST)
                .setPort(Integer.toString(ContainerUtils.CONTAINER_JMX_PORT)).setJmxUser(ContainerUtils.CONTAINER_USER)
                .setJmxPwd(ContainerUtils.CONTAINER_PWD).setPassPhrase("petals"));
        final Map<PortType, Integer> ports = new HashMap<>();
        ports.put(PortType.JMX, Integer.valueOf(ContainerUtils.CONTAINER_JMX_PORT));
        final Container container = new Container(ContainerUtils.CONTAINER_NAME, ContainerUtils.CONTAINER_HOST, ports,
                ContainerUtils.CONTAINER_USER, ContainerUtils.CONTAINER_PWD, ContainerUtils.CONTAINER_STATE);
        // TODO: The property name of the pass-phrase should be mutualized with the Petals µKernel
        container.addProperty("petals.topology.passphrase", "petals");
        this.adminApi.registerDomain().registerContainer(container);
        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final StringStreamShell shell = new StringStreamShell(pref);
        shell.addExtension(new PetalsAdminShellExtension());
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.execute(new String[] { "-" + Connect.YESFLAG_SHORT_OPTION });

        // Execute
        final Command command = new TopologyList();
        command.setShell(shell);
        command.execute(new String[] {});

        // Assertions
        final String output = shell.getOutput();
        assertTrue("Error occurs", shell.getError().isEmpty());
        assertFalse("No topology information is displayed !", output.isEmpty());
        assertTrue("JMX username not masked",
                shell.getOutput().contains("** JMX username: " + ContainerUtils.CONTAINER_USER));
        assertTrue("JMX password not masked",
                shell.getOutput().contains("** JMX password: " + ContainerUtils.CONTAINER_PWD));
    }

    /**
     * <p>
     * Test of the command "topology-list" against argument management, where:
     * </p>
     * <ul>
     * <li>a correct passphrase is defined at default connection level,</li>
     * <li>no passphrase is provided on command line,</li>
     * <li>an explicitly connection matching the default connection is used,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no critical topology information are printed.</li>
     * </ul>
     */
    @Test
    public void noPassPhraseGivenWithExplicitConnection() throws Exception {

        // Set the test environment
        this.tempPrefFile.addAlias(new DefaultAlias("my-alias").setHostName(ContainerUtils.CONTAINER_HOST)
                .setPort(Integer.toString(ContainerUtils.CONTAINER_JMX_PORT)).setJmxUser(ContainerUtils.CONTAINER_USER)
                .setJmxPwd(ContainerUtils.CONTAINER_PWD).setPassPhrase("petals"));
        final Map<PortType, Integer> ports = new HashMap<>();
        ports.put(PortType.JMX, Integer.valueOf(ContainerUtils.CONTAINER_JMX_PORT));
        final Container container = new Container(ContainerUtils.CONTAINER_NAME, ContainerUtils.CONTAINER_HOST, ports,
                ContainerUtils.CONTAINER_USER, ContainerUtils.CONTAINER_PWD, ContainerUtils.CONTAINER_STATE);
        // TODO: The property name of the pass-phrase should be mutualized with the Petals µKernel
        container.addProperty("petals.topology.passphrase", "petals");
        this.adminApi.registerDomain().registerContainer(container);
        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final StringStreamShell shell = new StringStreamShell(pref);
        shell.addExtension(new PetalsAdminShellExtension());
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.execute(new String[] { "-" + Connect.HOST_SHORT_OPTION, ContainerUtils.CONTAINER_HOST,
                "-" + Connect.PORT_SHORT_OPTION, Integer.toString(ContainerUtils.CONTAINER_JMX_PORT),
                "-" + Connect.USER_SHORT_OPTION, ContainerUtils.CONTAINER_USER, "-" + Connect.PWD_SHORT_OPTION,
                ContainerUtils.CONTAINER_PWD, "-" + Connect.YESFLAG_SHORT_OPTION });

        // Execute
        final Command command = new TopologyList();
        command.setShell(shell);
        command.execute(new String[] {});

        // Assertions
        final String output = shell.getOutput();
        assertTrue("Error occurs", shell.getError().isEmpty());
        assertFalse("No topology information is displayed !", output.isEmpty());
        assertTrue("JMX username not masked", shell.getOutput().contains("** JMX username: ********"));
        assertTrue("JMX password not masked", shell.getOutput().contains("** JMX password: ********"));
    }

    /**
     * <p>
     * Test of the command "topology-list" against argument management, where:
     * </p>
     * <ul>
     * <li>the passphrase is not provided on command line.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no critical topology information are printed.</li>
     * </ul>
     */
    @Test
    public void noPassPhraseGiven() throws Exception {

        // Set the test environment
        this.adminApi.registerDomain().registerContainer();
        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.execute(new String[] { "-" + Connect.HOST_SHORT_OPTION, ContainerUtils.CONTAINER_HOST,
                "-" + Connect.PORT_SHORT_OPTION, Integer.toString(ContainerUtils.CONTAINER_JMX_PORT),
                "-" + Connect.USER_SHORT_OPTION, ContainerUtils.CONTAINER_USER, "-" + Connect.PWD_SHORT_OPTION,
                ContainerUtils.CONTAINER_PWD });

        // Execute
        final Command command = new TopologyList();
        command.setShell(shell);
        command.execute(new String[] {});

        // Assertions
        final String output = shell.getOutput();
        assertFalse("No topology information is displayed !", output.isEmpty());
        assertTrue("JMX username not masked", shell.getOutput().contains("** JMX username: ********"));
        assertTrue("JMX password not masked", shell.getOutput().contains("** JMX password: ********"));
    }

    /**
     * <p>
     * Test of the command "topology-list" against argument management, where:
     * </p>
     * <ul>
     * <li>the passphrase is provided without its option.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void passphraseWithoutOption() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        shell.onConnectionEstablished();
        final Command command = new TopologyList();
        command.setShell(shell);
        try {
            command.execute(new String[] { "a-pass-phrase" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "topology-list" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedOption() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        shell.onConnectionEstablished();
        final Command command = new TopologyList();
        command.setShell(shell);
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "topology-list" against no domain found.
     * </p>
     * <p>
     * Expected results: an error occurs about no domain found.
     * </p>
     */
    @Test
    public void testError_0() throws Exception {

        // Set the environment:
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        shell.onConnectionEstablished();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute command & assertions
        final Command command = new TopologyList();
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + TopologyList.PASSPHRASE_SHORT_OPTION, "no-domain-found-passphrase" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandException e) {
            assertEquals("", "No Petals domain found", e.getMessage());
        }
    }

    /**
     * <p>
     * Test of the command "topology-list" against the option '-c' used without regexp.
     * </p>
     * <p>
     * Expected results: no error occurs, the name of the container on which we are connected is used.
     * </p>
     */
    @Test
    public void containerRegexpNotProvided() throws Exception {

        // Set the test environment
        this.adminApi.registerDomain(DomainUtils.createDomainSample());
        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl(DomainUtils.CONTAINER_00_HOST,
                DomainUtils.CONTAINER_00_JMX_PORT, "petals", "petals", null), false);

        // Check test env
        this.adminApi.getSingleton().connect(DomainUtils.CONTAINER_00_HOST, DomainUtils.CONTAINER_00_JMX_PORT, "petals",
                "petals");
        final ContainerAdministration containerAdminApi = this.adminApi.getSingleton().newContainerAdministration();
        final Domain domain = containerAdminApi.getTopology(".*", null, false);
        assertNotNull(domain);
        assertNotNull(domain.getContainers());
        assertEquals(2, domain.getContainers().size());

        // Execute test
        final Command command = new TopologyList();
        command.setShell(shell);
        command.execute(new String[] { "-" + TopologyList.CONTAINER_FILTER_SHORT_OPTION });

        // Check test result
        final String output = shell.getOutput();
        assertTrue("Error occurs", shell.getError().isEmpty());
        assertFalse("No topology information is displayed !", output.isEmpty());
        assertFalse("Other container information returned", output.contains(DomainUtils.CONTAINER_01_NAME));
    }

    /**
     * <p>
     * Test of the command "topology-list" against the option '-c' used with a regexp to match onlyt the container 0.
     * </p>
     * <p>
     * Expected results: no error occurs, the regexp match the container 0.
     * </p>
     */
    @Test
    public void containerRegexpProvidedAsContainerName() throws Exception {

        // Set the test environment
        this.adminApi.registerDomain(DomainUtils.createDomainSample());
        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl(DomainUtils.CONTAINER_00_HOST,
                DomainUtils.CONTAINER_00_JMX_PORT, "petals", "petals", null), false);

        // Check test env
        this.adminApi.getSingleton().connect(DomainUtils.CONTAINER_00_HOST, DomainUtils.CONTAINER_00_JMX_PORT, "petals",
                "petals");
        final ContainerAdministration containerAdminApi = this.adminApi.getSingleton().newContainerAdministration();
        final Domain domain = containerAdminApi.getTopology(".*", null, false);
        assertNotNull(domain);
        assertNotNull(domain.getContainers());
        assertEquals(2, domain.getContainers().size());
        boolean expectedContainerFound = false;
        for (final Container container : domain.getContainers()) {
            if (DomainUtils.CONTAINER_00_NAME.equals(container.getContainerName())) {
                expectedContainerFound = true;
            }
        }
        assertTrue(expectedContainerFound);

        // Execute test
        final Command command = new TopologyList();
        command.setShell(shell);
        command.execute(
                new String[] { "-" + TopologyList.CONTAINER_FILTER_SHORT_OPTION, DomainUtils.CONTAINER_00_NAME });

        // Check test result
        final String output = shell.getOutput();
        assertTrue("Error occurs", shell.getError().isEmpty());
        assertFalse("No topology information is displayed !", output.isEmpty());
        assertFalse("Other container information returned: " + output, output.contains(DomainUtils.CONTAINER_01_NAME));
    }

    /**
     * <p>
     * Test of the command "topology-list" against the option '-c' used with a regexp for all containers.
     * </p>
     * <p>
     * Expected results: no error occurs, the regexp match the expected containers.
     * </p>
     */
    @Test
    public void containerRegexpAllProvided() throws Exception {

        // Set the test environment
        this.adminApi.registerDomain(DomainUtils.createDomainSample());
        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl(DomainUtils.CONTAINER_00_HOST,
                DomainUtils.CONTAINER_00_JMX_PORT, "petals", "petals", null), false);

        // Check test env
        this.adminApi.getSingleton().connect(DomainUtils.CONTAINER_00_HOST, DomainUtils.CONTAINER_00_JMX_PORT, "petals",
                "petals");
        final ContainerAdministration containerAdminApi = this.adminApi.getSingleton().newContainerAdministration();
        final Domain domain = containerAdminApi.getTopology(".*", null, false);
        assertNotNull(domain);
        assertNotNull(domain.getContainers());
        assertEquals(2, domain.getContainers().size());

        // Execute test
        final Command command = new TopologyList();
        command.setShell(shell);
        command.execute(new String[] { "-" + TopologyList.CONTAINER_FILTER_SHORT_OPTION, ".*" });

        // Check test result
        final String output = shell.getOutput();
        assertTrue("Error occurs", shell.getError().isEmpty());
        assertFalse("No topology information is displayed !", output.isEmpty());
        assertTrue("Container 0 information not returned: " + output, output.contains(DomainUtils.CONTAINER_00_NAME));
        assertTrue("Container 1 information not returned: " + output, output.contains(DomainUtils.CONTAINER_01_NAME));

    }

    /**
     * <p>
     * Test of the command "topology-list" against the option '-a'.
     * </p>
     * <p>
     * Expected results: no error occurs, JBI artifacts deployed on each container of the topology are returned.
     * </p>
     */
    @Test
    public void listJbiArtifacts() throws Exception {

        // Set the test environment
        this.adminApi.registerDomain(DomainUtils.createDomainSample());
        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());

        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl(DomainUtils.CONTAINER_00_HOST,
                DomainUtils.CONTAINER_00_JMX_PORT, "petals", "petals", null), false);

        // Check test env
        this.adminApi.getSingleton().connect(DomainUtils.CONTAINER_00_HOST, DomainUtils.CONTAINER_00_JMX_PORT, "petals",
                "petals");
        final ContainerAdministration containerAdminApi = this.adminApi.getSingleton().newContainerAdministration();
        final Domain domain = containerAdminApi.getTopology(".*", DomainUtils.CONTAINER_00_PASSPHRASE, true);
        assertNotNull(domain);
        assertNotNull(domain.getContainers());
        assertEquals(2, domain.getContainers().size());
        for (final Container container : domain.getContainers()) {
            if (container.getContainerName().equals(DomainUtils.CONTAINER_00_NAME)) {
                assertEquals(1, container.getSharedLibraries().size());
                assertEquals(1, container.getComponents().size());
                assertEquals(0, container.getServiceAssemblies().size());
            } else if (container.getContainerName().equals(DomainUtils.CONTAINER_01_NAME)) {
                assertEquals(0, container.getSharedLibraries().size());
                assertEquals(2, container.getComponents().size());
                assertEquals(1, container.getServiceAssemblies().size());
            } else {
                fail("Unexpected container");
            }
        }

        // Execute test
        final Command command = new TopologyList();
        command.setShell(shell);
        command.execute(new String[] { "-" + TopologyList.ARTIFACTS_SHORT_OPTION,
                "-" + TopologyList.PASSPHRASE_SHORT_OPTION, DomainUtils.CONTAINER_00_PASSPHRASE });

        // Check test result
        final String output = shell.getOutput();
        assertTrue("Container '" + DomainUtils.CONTAINER_00_NAME + "' is missing",
                output.contains("\n* Container: " + DomainUtils.CONTAINER_00_NAME));
        assertTrue("Shared library '" + DomainUtils.SL_00_0_NAME + "' is missing",
                output.contains("\n** Shared libraries:\n*** " + DomainUtils.SL_00_0_NAME + " ("
                        + DomainUtils.SL_00_0_VERSION + ")\n"));
        assertTrue("Component '" + DomainUtils.BINDING_00_0 + "' is missing",
                output.contains("\n** Components:\n*** " + DomainUtils.BINDING_00_0));
        assertTrue("Container '" + DomainUtils.CONTAINER_01_NAME + "' is missing: " + output,
                output.contains("\n* Container: " + DomainUtils.CONTAINER_01_NAME));
        assertTrue("Component '" + DomainUtils.ENGINE_01_0 + "' is missing",
                output.contains("\n*** " + DomainUtils.ENGINE_01_0));
        assertTrue("Component '" + DomainUtils.ENGINE_01_1 + "' is missing",
                output.contains("\n*** " + DomainUtils.ENGINE_01_1));
        assertTrue("Service assembly '" + DomainUtils.SA_01_0 + "' is missing",
                output.contains("\n** Service assemblies:\n*** " + DomainUtils.SA_01_0));
    }

    /**
     * <p>
     * Test of the command "topology-list" against the option '-a'.
     * </p>
     * <p>
     * Expected results: an error occurs about the missing pass-phrase.
     * </p>
     */
    @Test
    public void listJbiArtifactsWithoutPassPhrase() throws Exception {

        // Set the test environment
        this.adminApi.registerDomain(DomainUtils.createDomainSample());
        final StringStreamShell shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());

        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl(DomainUtils.CONTAINER_00_HOST,
                DomainUtils.CONTAINER_00_JMX_PORT, "petals", "petals", null), false);

        // Execute test
        final Command command = new TopologyList();
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + TopologyList.ARTIFACTS_SHORT_OPTION });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "topology-list" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new TopologyList();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
