/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.endsWith;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.URL_LONG_OPTION;

import java.io.File;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactTypeIsNeededException;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.Main;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.ConnectionCommand;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingYesFlagException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.InvalidConnectionParameterException;
import org.ow2.petals.cli.api.shell.ShellExtension;
import org.ow2.petals.cli.base.junit.shell.PetalsInteractiveCliUtils;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.shell.PetalsCli;
import org.ow2.petals.cli.shell.ShellFactory;
import org.ow2.petals.cli.shell.extension.PetalsAdminShellExtension;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class UndeployTest extends AbstractArtifactCommandTest {

    /**
     * The shell {@link PetalsCli} establishes automatically a connection. Here, the default connection parameters.
     */
    private final ConnectionParameters defaultConnectionParameters;

    private final static String EXPECTED_CMD_USAGE = String.format(
            "usage: undeploy [-a <artifact-id>] [-b <artifacts-directory>] [-t%n       <artifact-type>] [-u <url>] [-v <version>] [-y]%n");

    @Rule
    public TemporaryFolder folder = new TemporaryFolder();

    public UndeployTest() {
        try {
            this.defaultConnectionParameters = new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals",
                    "petals", null);
        } catch (final UnknownHostException e) {
            // NOP, the exception can not occur
            throw new RuntimeException("An unexpected exception occurs", e);
        } catch (final InvalidConnectionParameterException e) {
            // NOP, the exception can not occur
            throw new RuntimeException("An unexpected exception occurs", e);
        }
    }

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws Exception {
        final Undeploy command = new Undeploy();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertEquals("Unexpected command usage", EXPECTED_CMD_USAGE, usage);
    }

    /**
     * <p>
     * Test of the command "undeploy" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_00() throws Exception {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" against argument management, where:
     * </p>
     * <ul>
     * <li>undeployment by url is provided,</li>
     * <li>bulk mode is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_01() throws Exception {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + Constants.CommonOption.URL_SHORT_OPTION, "file:///tmp",
                    "-" + Undeploy.BULK_SHORT_OPTION });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" against argument management, where:
     * </p>
     * <ul>
     * <li>undeployment by artifact is provided,</li>
     * <li>undeployment by url is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_02() throws Exception {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, "my-artifact",
                    "-" + Constants.CommonOption.URL_SHORT_OPTION, "file:///tmp" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" against argument management, where:
     * </p>
     * <ul>
     * <li>undeployment by artifact is provided,</li>
     * <li>bulk mode is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_03() throws Exception {

        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, "my-artifact",
                    "-" + Undeploy.BULK_SHORT_OPTION });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" against an undeployment of all artifacts, where:
     * </p>
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>no 'yes' flag,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing yes flag.
     * </p>
     */
    @Test()
    public void testUndeployBulkError_00() throws Exception {

        final Undeploy command = new Undeploy();
        final PetalsCli shell = new PetalsCli(new String[] { command.getName(), "-" + Undeploy.BULK_SHORT_OPTION },
                false, false, null, new ShellExtension[] { new PetalsAdminShellExtension() }, null);
        shell.registersCommand(command);
        command.setShell(shell);
        final ConnectionCommand connect = new Connect();
        connect.setShell(shell);
        shell.registersConnectionCommand(connect);

        shell.setConnectionParameters(this.defaultConnectionParameters);
        shell.run();

        assertTrue("It's not the expected error message.",
                systemErr.getLog().contains(CommandMissingYesFlagException.MISSING_YESFLAG));

    }

    /**
     * <p>
     * Test of the command "undeploy" against an undeployment of all artifacts, where:
     * </p>
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>with the yes flag at command line level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * <p>
     * Expected results: No error occurs about missing yes flag
     * </p>
     */
    @Test
    public void testUndeployBulk_00() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();

        // Set the Petals ESB environment
        this.deployStartArtifact(COMPONENT_URL, SA_URL);
        assertEquals("Unexpected deployed artifact number", 10, admin.listArtifacts().size());

        runCliBlocking("-" + Connect.HOST_SHORT_OPTION, "localhost", "-" + Connect.PORT_SHORT_OPTION, "7700",
                "-" + Connect.USER_SHORT_OPTION, "petals", "-" + Connect.PWD_SHORT_OPTION, "petals",
                "-" + ShellFactory.YESFLAG_SHORT_OPTION, "-" + ShellFactory.COMMAND_SHORT_OPTION, "--",
                new Undeploy().getName(), "-" + Undeploy.BULK_SHORT_OPTION);

        assertTrue("An error message is print", systemErr.getLog().isEmpty());
        assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());

    }

    /**
     * <p>
     * Test of the command "undeploy" against an undeployment of all artifacts, where:
     * </p>
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * <p>
     * Expected results: No error occurs about missing yes flag
     * </p>
     */
    @Test
    public void testUndeployBulk_01() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();

        // Set the Petals ESB environment
        this.deployStartArtifact(COMPONENT_URL, SA_URL);
        assertEquals("Unexpected deployed artifact number", 10, admin.listArtifacts().size());

        runCliBlocking("-" + Connect.HOST_SHORT_OPTION, "localhost", "-" + Connect.PORT_SHORT_OPTION, "7700",
                "-" + Connect.USER_SHORT_OPTION, "petals", "-" + Connect.PWD_SHORT_OPTION, "petals",
                "-" + ShellFactory.COMMAND_SHORT_OPTION, "--", new Undeploy().getName(),
                "-" + Undeploy.BULK_SHORT_OPTION, "-" + Undeploy.YESFLAG_SHORT_OPTION);

        assertTrue("An error message is print", systemErr.getLog().isEmpty());
        assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());

    }

    /**
     * <p>
     * Test of the command "undeploy" against an undeployment of all artifacts, where:
     * </p>
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>a confirmation message is displayed,</li>
     * <li>after confirmation, the command is executed displaying the prompt</li>
     * </ul>
     */
    @Test
    public void testUndeployBulkError_02() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            // Set the Petals ESB environment
            this.deployStartArtifact(COMPONENT_URL, SA_URL);
            assertEquals("Unexpected deployed artifact number", 10, admin.listArtifacts().size());

            runCli("-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect -h localhost -n 7700 -u petals -p petals");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            // Enter the command
            systemIn.writeLine(new Undeploy().getName() + " -b");

            // Wait the execution of the command
            checkOut(endsWith(Undeploy.UNDEPLOY_ALL_CONFIRMATION_MSG));

            // Enter confirmation
            systemIn.writeLine("y");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

            assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" against an undeployment of all artifacts, where:
     * </p>
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>with the yes flag at command line level,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no confirmation message is displayed,</li>
     * <li>the command is executed displaying the prompt</li>
     * </ul>
     */
    @Test
    public void testUndeployBulk_02() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            // Set the Petals ESB environment
            this.deployStartArtifact(COMPONENT_URL, SA_URL);
            assertEquals("Unexpected deployed artifact number", 10, admin.listArtifacts().size());

            runCli("-" + ShellFactory.YESFLAG_SHORT_OPTION, "-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect -h localhost -n 7700 -u petals -p petals");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            // Enter the command
            systemIn.writeLine(new Undeploy().getName() + " -b");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

            assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" against an undeployment of all artifacts, where:
     * </p>
     * <ul>
     * <li>bulk mode enable,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no confirmation message is displayed,</li>
     * <li>the command is executed displaying the prompt,</li>
     * <li>no more artifacts remain deployed.</li>
     * </ul>
     */
    @Test
    public void testUndeployBulk_03() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            // Set the Petals ESB environment
            this.deployStartArtifact(COMPONENT_URL, SA_URL);
            assertEquals("Unexpected deployed artifact number", 10, admin.listArtifacts().size());

            runCli("-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect -h localhost -n 7700 -u petals -p petals");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            // Enter the command
            systemIn.writeLine(new Undeploy().getName() + " -b -y");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:7700>"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

            assertEquals("Artifact is always deployed", 0, admin.listArtifacts().size());

        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * <p>
     * Test of the command "undeploy" against an undeployment of all artifacts, where:
     * </p>
     * <ul>
     * <li>bulk mode enable using a directory url,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * <p>
     * Expected results: Specified artifacts are correctly undeployed
     * </p>
     */
    @Test
    public void testUndeployBulk_04() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();

        // Set the Petals ESB environment
        final URL jbiCompXslt = Thread.currentThread().getContextClassLoader().getResource("jbi-component-xslt.zip");
        assertNotNull("Test component XSLT not found", jbiCompXslt);
        this.deployStartArtifact(COMPONENT_URL, SA_URL, jbiCompXslt);
        assertEquals("Unexpected deployed artifact number", 11, admin.listArtifacts().size());

        // Create a directory to undeploy a list of artifacts
        final File undeploymentDir = this.folder.newFolder("undeployment-dir");
        FileUtils.copyURLToFile(COMPONENT_URL, new File(undeploymentDir, COMPONENT_FILENAME));
        FileUtils.copyURLToFile(SA_URL, new File(undeploymentDir, SA_PERMISSION_FILENAME));

        runCliBlocking("-" + Connect.HOST_SHORT_OPTION, "localhost", "-" + Connect.PORT_SHORT_OPTION, "7700",
                "-" + Connect.USER_SHORT_OPTION, "petals", "-" + Connect.PWD_SHORT_OPTION, "petals",
                "-" + ShellFactory.COMMAND_SHORT_OPTION, "--", new Undeploy().getName(),
                "-" + Undeploy.BULK_SHORT_OPTION, undeploymentDir.toURI().toURL().toString(),
                "-" + Undeploy.YESFLAG_SHORT_OPTION);

        assertTrue("An error message is print", systemErr.getLog().isEmpty());
        final List<Artifact> artifactsList = admin.listArtifacts();
        assertEquals("It should remain 1 artifact deployed", 1, artifactsList.size());
        final Artifact artifact = artifactsList.get(0);
        assertEquals("Unexpected remaining artifact", "petals-se-xslt", artifact.getName());

    }

    @Test()
    public void testUndeployOneArtifactsByFileURL() throws Exception {
        final ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_URL);

        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "--" + URL_LONG_OPTION, SA_URL.toString() });
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        assertTrue(artifacts.get(0) instanceof Component);
    }

    @Test()
    public void testUndeployOneArtifactsByMavenURL() throws Exception {
        final ArtifactAdministration admin = getArtifactAdmin();
        final URL componentURL = new URL("mvn:org.ow2.petals/petals-bc-soap/4.4.2/zip");
        this.deployStartArtifact(componentURL);

        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "--" + URL_LONG_OPTION, componentURL.toString() });
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(0, artifacts.size());
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by artifact type and artifact identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>two JBI artifacts are deployed with different identifiers.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly undeployed</li>
     * </ul>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_0() throws Exception {
        final ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_URL);

        Artifact a = admin.getArtifactInfo(SA_URL);
        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "--" + ArtifactBasedCommand.ARTIFACT_LONG_OPTION, a.getName(),
                "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, a.getType() });
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        assertTrue(artifacts.get(0) instanceof Component);

    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by artifact type and artifact identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>two JBI artifacts are deployed with different identifiers.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly undeploy</li>
     * </ul>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_1() throws Exception {
        final ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_URL);

        Artifact a = admin.getArtifactInfo(SA_URL);
        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "--" + ArtifactBasedCommand.ARTIFACT_LONG_OPTION, a.getName() });
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        assertTrue(artifacts.get(0) instanceof Component);

    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by its type and its identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing,</li>
     * <li>two JBI artifacts are deployed with the same identifier,</li>
     * <li>one JBI artifact is deployed with a different identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about the missing artifact type</li>
     * <li>the artifacts are not undeployed</li>
     * </ul>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_2() throws Exception {
        final ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_URL, SA_SOAP_ID_URL);

        Artifact a = admin.getArtifactInfo(COMPONENT_URL);
        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, a.getName() });
            fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactTypeIsNeededException",
                    e.getCause() instanceof ArtifactTypeIsNeededException);
        }

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(19, artifacts.size());

    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by its type and its identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>the right artifact is undeployed</li>
     * <li>the other artifact is not undeployed</li>
     * </ul>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_3() throws Exception {
        ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_SOAP_ID_URL);

        Artifact a = admin.getArtifactInfo(SA_SOAP_ID_URL);
        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, a.getName(),
                "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, a.getType() });
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        assertTrue(artifacts.get(0) instanceof Component);

    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by its type and its identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about bad argument number</li>
     * <li>the artifacts are always deployed</li>
     * </ul>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testUndeployOneArtifactsByTypeAndName_4() throws Exception {
        ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_SOAP_ID_URL);

        Artifact a = admin.getArtifactInfo(SA_SOAP_ID_URL);
        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, a.getType() });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandBadArgumentNumberException e) {
            // Expected exception
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(10, artifacts.size());
            throw e;
        }

    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by its type and its identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifacts are always deployed</li>
     * </ul>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_5() throws Exception {
        ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_SOAP_ID_URL);

        Artifact a = admin.getArtifactInfo(SA_SOAP_ID_URL);
        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            assertFalse("The artifact is a SE", "SE".equals(a.getType()));
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, a.getName(),
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, "SE" });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactNotFoundException",
                    e.getCause() instanceof ArtifactNotFoundException);
        }
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(10, artifacts.size());

    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by artifact type and artifact identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifact is always deployed</li>
     * </ul>
     */
    @Test()
    public void testUndeployOneArtifactsByTypeAndName_6() throws Exception {
        ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_URL);

        Artifact a = admin.getArtifactInfo(SA_URL);
        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            assertFalse("The artifact is a SE", "SE".equals(a.getType()));
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, a.getName(),
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, "SE" });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactNotFoundException",
                    e.getCause() instanceof ArtifactNotFoundException);
        }
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(10, artifacts.size());
    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by artifact type and artifact identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>an error occurs about bad argument number</li>
     * <li>the artifact is always deployed</li>
     * </ul>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testUndeployOneArtifactsByTypeAndName_7() throws Exception {
        ArtifactAdministration admin = getArtifactAdmin();
        this.deployStartArtifact(COMPONENT_URL, SA_URL);

        Artifact a = admin.getArtifactInfo(SA_URL);
        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, a.getType() });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandBadArgumentNumberException e) {
            // Expected exception
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(10, artifacts.size());
            throw e;
        }

    }

    /**
     * <p>
     * Test of the command "undeploy" to undeploy an artifact identified by artifact type and artifact identifier where:
     * </p>
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a artifact type option is provided without value,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about artifact not found
     * </p>
     */
    @Test
    public void testUndeployOneArtifactsByTypeAndName_8() throws Exception {
        this.deployStartArtifact(COMPONENT_URL);

        Component a = (Component) ArtifactUtils.createArtifact(COMPONENT_URL);

        Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        String identifier = a.getName();
        try {
            assertFalse("The artifact is a SE", "SE".equals(a.getType()));
            command.execute(new String[] { "-" + Undeploy.ARTIFACT_SHORT_OPTION, identifier,
                    "-" + Undeploy.TYPE_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue("The option '-" + Undeploy.TYPE_SHORT_OPTION + "' has a missing argument.",
                    Undeploy.TYPE_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(Undeploy.TYPE_SHORT_OPTION));
        }
    }

    @Test
    public void testUndeploySharedLibraryWithSLVersionFilter() throws Exception {
        this.deployStartArtifact(slURL, sl2URL, COMPONENT_URL);

        final Artifact slArtifact = ArtifactUtils.createArtifact(this.slURL);
        final Undeploy command = new Undeploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + Undeploy.ARTIFACT_SHORT_OPTION, slArtifact.getName(),
                "-" + Undeploy.VERSION_SHORT_OPTION, slArtifact.getVersion() });

        assertEquals(2, getArtifactAdmin().listArtifacts().size());

    }

    /**
     * <p>
     * Test of the command "undeploy" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new Undeploy();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
