/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.completer;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.api.artifact.ArtifactState.State;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Component.ComponentType;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.junit.PetalsAdministrationApi;
import org.ow2.petals.admin.junit.exception.MoreThanOneContainerRegisteredException;
import org.ow2.petals.admin.junit.exception.NoContainerRegisteredException;
import org.ow2.petals.admin.junit.exception.NoDomainRegisteredException;

import jline.console.completer.Completer;

/**
 * Unit tests of {@link AllArtifactNameCompleter}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ArtifactNameCompleterTest {

    @Rule
    public PetalsAdministrationApi petalsAdminApi = new PetalsAdministrationApi();

    @Test
    public void testComplete() throws DuplicatedServiceException, MissingServiceException,
            NoContainerRegisteredException, NoDomainRegisteredException, MoreThanOneContainerRegisteredException {

        this.petalsAdminApi.registerDomain().registerContainer();

        final String slOneName = "shared-library";
        final String slOneVersion = "shared-library-version";
        this.petalsAdminApi.registerArtifact(new SharedLibrary(slOneName, slOneVersion));

        final String componentOneName = "started-binding-component";
        final Component componentOne = new Component(componentOneName, ComponentType.BC, State.STARTED);
        this.petalsAdminApi.registerArtifact(componentOne);

        final String componentTwoName = "stopped-binding-component";
        final Component componentTwo = new Component(componentTwoName, ComponentType.SE, State.STOPPED);
        this.petalsAdminApi.registerArtifact(componentTwo);

        final String componentThreeName = "loaded-binding-component";
        final Component componentThree = new Component(componentThreeName, ComponentType.BC, State.LOADED);
        this.petalsAdminApi.registerArtifact(componentThree);

        final String componentFourName = "shutdown-binding-component";
        final Component componentFour = new Component(componentFourName, ComponentType.BC, State.SHUTDOWN);
        this.petalsAdminApi.registerArtifact(componentFour);

        final String componentFiveName = "unknown-binding-component";
        final Component componentFive = new Component(componentFiveName, ComponentType.BC, State.UNKNOWN);
        this.petalsAdminApi.registerArtifact(componentFive);

        final String saOneName = "started-service-assembly";
        final ServiceAssembly saOne = new ServiceAssembly(saOneName, State.STARTED);
        final String suOneName = "service-unit";
        final ServiceUnit suOne = new ServiceUnit(suOneName, componentOneName);
        saOne.addServiceUnit(suOne);
        this.petalsAdminApi.registerArtifact(saOne);

        final String saTwoName = "stopped-service-assembly";
        final ServiceAssembly saTwo = new ServiceAssembly(saTwoName, State.STOPPED);
        this.petalsAdminApi.registerArtifact(saTwo);

        final String saThreeName = "unknown-service-assembly";
        final ServiceAssembly saThree = new ServiceAssembly(saThreeName, State.UNKNOWN);
        this.petalsAdminApi.registerArtifact(saThree);

        final List<CharSequence> allArtifacts = new ArrayList<>();
        allArtifacts.add(slOneName);
        allArtifacts.add(componentOneName);
        allArtifacts.add(componentTwoName);
        allArtifacts.add(componentThreeName);
        allArtifacts.add(componentFourName);
        allArtifacts.add(componentFiveName);
        allArtifacts.add(saOneName);
        allArtifacts.add(saTwoName);
        allArtifacts.add(saThreeName);
        allArtifacts.add(suOneName);

        final List<CharSequence> artifactsThatCanBeStopped = new ArrayList<>();
        artifactsThatCanBeStopped.add(componentOneName);
        artifactsThatCanBeStopped.add(saOneName);

        final List<CharSequence> artifactsThatCanBeStarted = new ArrayList<>();
        artifactsThatCanBeStarted.add(componentTwoName);
        artifactsThatCanBeStarted.add(componentFourName);
        artifactsThatCanBeStarted.add(saTwoName);

        final List<CharSequence> candidates = new ArrayList<>();

        final PetalsAdministration petalsAdministration = this.petalsAdminApi.newPetalsAdministration();

        // Check AllArtifactNameCompleter
        final Completer allArtifactNameCompleter = new AllArtifactNameCompleter(
                petalsAdministration.newArtifactAdministration());
        allArtifactNameCompleter.complete("", 0, candidates);
        assertNotNull(candidates);
        assertEquals("AllArtifactNameCompleter: Unexpected candidates", new HashSet<>(allArtifacts),
                new HashSet<>(candidates));

        // Check StartedArtifactNameCompleter
        candidates.clear();
        final Completer startedArtifactNameCompleter = new StartedArtifactNameCompleter(
                petalsAdministration.newArtifactAdministration());
        startedArtifactNameCompleter.complete("", 0, candidates);
        assertNotNull(candidates);
        assertEquals("StartedArtifactNameCompleter: Unexpected candidates",
                new HashSet<>(artifactsThatCanBeStopped), new HashSet<>(candidates));

        // Check StoppedArtifactNameCompleter
        candidates.clear();
        final Completer stoppedArtifactNameCompleter = new StoppedArtifactNameCompleter(
                petalsAdministration.newArtifactAdministration());
        stoppedArtifactNameCompleter.complete("", 0, candidates);
        assertNotNull(candidates);
        assertEquals("StoppedArtifactNameCompleter: Unexpected candidates",
                new HashSet<>(artifactsThatCanBeStarted), new HashSet<>(candidates));
    }

}
