/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.completer;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.shell.Shell;

import jline.console.completer.Completer;
import jline.console.completer.StringsCompleter;

/**
 * Unit tests of {@link CommandCompleter}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class CommandCompleterTest {

    /**
     * Prefix of the unit test commands
     */
    private static final String MY_COMMAND_NAME_PREFIX = "my-command-";

    /**
     * The name of the unit test command
     */
    private static final String MY_COMMAND_ONE_NAME = MY_COMMAND_NAME_PREFIX + "one";

    /**
     * The name of the unit test command
     */
    private static final String MY_COMMAND_TWO_NAME = MY_COMMAND_NAME_PREFIX + "two";

    /**
     * An abstract implementation of {@link Command} for unit test purpose.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    private abstract class AbstractCommandToComplete implements Command {

        @Override
        public Options getOptions() {
            return null;
        }

        @Override
        public void resetOptions() {
            // NOP
        }

        @Override
        public String getOptionsDescription() {
            return null;
        }

        @Override
        public String getUsage() {
            return null;
        }

        @Override
        public String getDescription() {
            return null;
        }

        @Override
        public Completer getDefaultCompleter() {
            return null;
        }

        @Override
        public Map<String, Completer> getOptionCompleters() {
            return null;
        }

        @Override
        public Shell getShell() {
            return null;
        }

        @Override
        public void setShell(final Shell shell) {
            // NOP
        }

        @Override
        public void execute(final String[] args) throws CommandException {
            // NOP
        }

        @Override
        public boolean isConnectionRequired() {
            return false;
        }
    }

    /**
     * A {@link Command} for unit test purpose.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    private class MyCommandOne extends AbstractCommandToComplete {

        @Override
        public String getName() {
            return MY_COMMAND_ONE_NAME;
        }
    }

    /**
     * Another {@link Command} for unit test purpose.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    private class MyCommandTwo extends AbstractCommandToComplete {

        @Override
        public String getName() {
            return MY_COMMAND_TWO_NAME;
        }
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against a command list set to <code>null</code>.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no completion done.</li>
     * </ul>
     */
    @Test
    public void withCommandListNull() {
        final Completer commandCompleter = new CommandCompleter(null);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        assertEquals(-1, commandCompleter.complete("mycomm", 0, candidates));
        assertEquals(0, candidates.size());
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against an empty command list.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no completion done.</li>
     * </ul>
     */
    @Test
    public void withCommandListEmpty() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        assertEquals(-1, commandCompleter.complete("mycomm", 0, candidates));
        assertEquals(0, candidates.size());
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of a command name where several commands match the
     * entered buffer.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>several commands are proposed.</li>
     * </ul>
     */
    @Test
    public void completionCommandNameSeveralCommandsMatch() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne());
        commands.put(MY_COMMAND_TWO_NAME, new MyCommandTwo());
        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        assertEquals(0, commandCompleter.complete(MY_COMMAND_NAME_PREFIX, MY_COMMAND_NAME_PREFIX.length(), candidates));
        assertEquals(2, candidates.size());
        assertTrue(candidates.contains(MY_COMMAND_ONE_NAME));
        assertTrue(candidates.contains(MY_COMMAND_TWO_NAME));
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of a command name where only one command matches the
     * entered buffer.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the command name is completed suffixed with a space.</li>
     * </ul>
     */
    @Test
    public void completionCommandNameOnlyOneCommandMatch() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne());
        commands.put(MY_COMMAND_TWO_NAME, new MyCommandTwo());
        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        assertEquals(0, commandCompleter.complete(MY_COMMAND_NAME_PREFIX + "o", MY_COMMAND_NAME_PREFIX.length() + 1,
                candidates));
        assertEquals(1, candidates.size());
        assertEquals(MY_COMMAND_ONE_NAME + " ", candidates.get(0).toString());
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of a command name where the full command name is
     * already entered.
     * </p>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>the command name is completed.</li>
     * </ul>
     */
    @Test
    public void completionFullCommandName() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne());
        commands.put(MY_COMMAND_TWO_NAME, new MyCommandTwo());
        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        assertEquals(0, commandCompleter.complete(MY_COMMAND_ONE_NAME, MY_COMMAND_ONE_NAME.length(), candidates));
        assertEquals(1, candidates.size());
        assertEquals(MY_COMMAND_ONE_NAME + " ", candidates.get(0).toString());
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of command arguments where:
     * </p>
     * <ul>
     * <li>no beginning of argument is given,</li>
     * <li>the default completer is set to null</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no argument completion</li>
     * </ul>
     */
    @Test
    public void completionCommandArgsDefaultCompleterNull() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne());
        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        final String buffer = MY_COMMAND_ONE_NAME + " ";
        assertEquals(-1, commandCompleter.complete(buffer, buffer.length(), candidates));
        assertEquals(0, candidates.size());
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of command arguments where:
     * </p>
     * <ul>
     * <li>no beginning of argument is given,</li>
     * <li>the default completer is correctly set</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>propositions are given for argument</li>
     * </ul>
     */
    @Test
    public void completionCommandArgsDefaultCompleter() {
        final String myArgument = "my-argument";
        final Map<String, Command> commands = new HashMap<String, Command>();
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne() {

            @Override
            public Completer getDefaultCompleter() {
                return new StringsCompleter(myArgument);
            }
        });
        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        final String buffer = MY_COMMAND_ONE_NAME + " ";
        assertEquals(buffer.length(), commandCompleter.complete(buffer, buffer.length(), candidates));
        assertEquals(1, candidates.size());
        assertEquals(myArgument + " ", candidates.get(0));
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of command arguments where:
     * </p>
     * <ul>
     * <li>no beginning of argument is given,</li>
     * <li>the command has no options (options completer is set to null, options are set to null)</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no argument completion</li>
     * </ul>
     */
    @Test
    public void completionCommandArgsNoOption() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne());
        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        final String buffer = MY_COMMAND_ONE_NAME + " -";
        assertEquals(-1, commandCompleter.complete(buffer, buffer.length(), candidates));
        assertEquals(0, candidates.size());
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of command arguments where:
     * </p>
     * <ul>
     * <li>no beginning of argument is given,</li>
     * <li>the command has no options (options completer is set to null, options are set to null)</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no argument completion</li>
     * </ul>
     */
    @Test
    public void completionCommandArgsOptionsCompleterNull() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        final String shortOption = "o";
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne() {

            @Override
            public Options getOptions() {
                return new Options().addOption(Option.builder(shortOption).build());
            }
        });

        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        final String buffer = MY_COMMAND_ONE_NAME + " -";
        commandCompleter.complete(buffer, buffer.length(), candidates);
        assertEquals(1, candidates.size());
        assertEquals("-" + shortOption + " ", candidates.get(0));
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of command argument options where:
     * </p>
     * <ul>
     * <li>no beginning of argument is given,</li>
     * <li>the command has an option</li>
     * <li>the option has argument</li>
     * <li>the option completer is set to null</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no argument completion</li>
     * </ul>
     */
    @Test
    public void completionCommandOptionWithArgOptionsCompleterNull() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        final String shortOption = "o";
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne() {

            @Override
            public Options getOptions() {
                return new Options().addOption(Option.builder(shortOption).hasArg(true).build());
            }
        });

        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        final String buffer = MY_COMMAND_ONE_NAME + " -" + shortOption + " ";
        commandCompleter.complete(buffer, buffer.length(), candidates);
        assertEquals(0, candidates.size());
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of command argument options where:
     * </p>
     * <ul>
     * <li>no beginning of argument is given,</li>
     * <li>the command has an option</li>
     * <li>the option has argument</li>
     * <li>the option completer is empty</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no argument completion</li>
     * </ul>
     */
    @Test
    public void completionCommandOptionWithArgOptionsCompleterEmpty() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        final String shortOption = "o";
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne() {

            @Override
            public Map<String, Completer> getOptionCompleters() {
                return new HashMap<String, Completer>();
            }

            @Override
            public Options getOptions() {
                return new Options().addOption(Option.builder(shortOption).hasArg(true).build());
            }
        });

        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        final String buffer = MY_COMMAND_ONE_NAME + " -" + shortOption + " ";
        commandCompleter.complete(buffer, buffer.length(), candidates);
        assertEquals(0, candidates.size());
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of command argument options where:
     * </p>
     * <ul>
     * <li>no beginning of argument is given,</li>
     * <li>the command has an option</li>
     * <li>the option has argument</li>
     * <li>an option completer is correctly set</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no argument completion</li>
     * </ul>
     */
    @Test
    public void completionCommandOptionWithArgOptionsCompleter() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        final String shortOption = "o";
        final String argValue = "argValue";
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne() {

            @Override
            public Map<String, Completer> getOptionCompleters() {
                final Map<String, Completer> optionCompleter = new HashMap<String, Completer>();
                optionCompleter.put(shortOption, new StringsCompleter(argValue));
                return optionCompleter;
            }

            @Override
            public Options getOptions() {
                return new Options().addOption(Option.builder(shortOption).hasArg(true).build());
            }
        });

        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        final String buffer = MY_COMMAND_ONE_NAME + " -" + shortOption + " " + argValue.substring(0, 1);
        commandCompleter.complete(buffer, buffer.length(), candidates);
        assertEquals(1, candidates.size());
        assertEquals(argValue + " ", candidates.get(0));
    }

    /**
     * <p>
     * Check the {@link CommandCompleter} against the completion of command argument options where:
     * </p>
     * <ul>
     * <li>no beginning of argument is given,</li>
     * <li>the command has an option with only long form</li>
     * <li>the option has argument</li>
     * <li>an option completer is correctly set</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>no argument completion</li>
     * </ul>
     */
    @Test
    public void completionCommandLongOptionWithArgOptionsCompleter() {
        final Map<String, Command> commands = new HashMap<String, Command>();
        final String longOption = "long-option";
        final String argValue = "argValue";
        commands.put(MY_COMMAND_ONE_NAME, new MyCommandOne() {

            @Override
            public Map<String, Completer> getOptionCompleters() {
                final Map<String, Completer> optionCompleter = new HashMap<String, Completer>();
                optionCompleter.put(longOption, new StringsCompleter(argValue));
                return optionCompleter;
            }

            @Override
            public Options getOptions() {
                return new Options().addOption(Option.builder().hasArg(true).longOpt(longOption).build());
            }
        });

        final Completer commandCompleter = new CommandCompleter(commands);
        final List<CharSequence> candidates = new ArrayList<CharSequence>();
        final String buffer = MY_COMMAND_ONE_NAME + " --" + longOption + " " + argValue.substring(0, 1);
        commandCompleter.complete(buffer, buffer.length(), candidates);
        assertEquals(1, candidates.size());
        assertEquals(argValue + " ", candidates.get(0));
    }

}
