/**
 * Copyright (c) 2015-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;

/**
 * Unit tests of {@link Utils}
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class UtilsTest {

    @Rule
    public final TemporaryFolder tempFolder = new TemporaryFolder();

    /**
     * Unit tests of {@link Utils#toURL(String)} and {@link Utils#toFile(String)}
     */
    @Test
    public void toURLtoFile() throws URISyntaxException, MalformedURLException {

        final String anAbsoluteFile = "/tmp/myfile.txt";
        final String aDir = "/tmp/dir";
        final String httpServer = "http://localhost";
        final String homeDirFile = "~/myfile.txt";
        final String aDirWithSpace = "/tmp/a dir with space";
        final String anAbsoluteFileWithSpace = "/tmp/a file with space.txt";
        
        assertEquals(new URL("file://" + anAbsoluteFile), Utils.toURL(anAbsoluteFile));
        assertEquals(new File(anAbsoluteFile), Utils.toFile(anAbsoluteFile));
        assertNull(Utils.toFile("file://" + anAbsoluteFile));
        assertEquals(new URL("file://" + anAbsoluteFileWithSpace.replace(" ", "%20")),
                Utils.toURL(anAbsoluteFileWithSpace));
        assertEquals(new File(anAbsoluteFileWithSpace), Utils.toFile(anAbsoluteFileWithSpace));
        assertEquals(new URL("file://" + aDir), Utils.toURL(aDir));
        assertEquals(new File(aDir), Utils.toFile(aDir));
        assertEquals(new URL("file://" + aDirWithSpace.replace(" ", "%20")), Utils.toURL(aDirWithSpace));
        assertEquals(new File(aDirWithSpace), Utils.toFile(aDirWithSpace));
        assertEquals(new URL(httpServer), Utils.toURL(httpServer));
        assertNull(Utils.toFile(httpServer));
        assertEquals(new URL("file://" + System.getProperty("user.home") + "/myfile.txt"), Utils.toURL(homeDirFile));
        assertEquals(new File(System.getProperty("user.home") + "/myfile.txt"), Utils.toFile(homeDirFile));
        
    }

    /**
     * Unit tests of {@link Utils#getUrls(java.io.File)} where the directory contains files (zip files and not zip
     * files)
     */
    @Test
    public void getUrls_WithFiles() throws IOException {

        final File rootDir = this.tempFolder.newFolder();
        final String fileName1 = "file-1.zip";
        final String fileName2 = "file-2.zip";
        final File file1 = new File(rootDir, fileName1);
        final File file2 = new File(rootDir, fileName2);
        final File file3 = new File(rootDir, "file-3");
        assertTrue(file1.createNewFile());
        assertTrue(file2.createNewFile());
        assertTrue(file3.createNewFile());

        final URL[] urls = Utils.getUrls(rootDir);

        assertEquals(2, urls.length);
        boolean file1found = false;
        boolean file2found = false;
        for (final URL url : urls) {
            if (url.getPath().equals(file1.getAbsolutePath())) {
                file1found = true;
            } else if (url.getPath().equals(file2.getAbsolutePath())) {
                file2found = true;
            } else {
                fail("unexpected file found");
            }
        }
        assertTrue(file1found);
        assertTrue(file2found);
    }

    /**
     * Unit tests of {@link Utils#getUrls(java.io.File)} where the directory contains no file
     */
    @Test
    public void getUrls_WithNoFile() throws IOException {

        final File rootDir = this.tempFolder.newFolder();
        final URL[] urls = Utils.getUrls(rootDir);
        assertEquals(0, urls.length);
    }

    /**
     * Unit tests of {@link Utils#getUrls(java.io.File)} where the directory does not exist
     */
    @Test(expected = FileNotFoundException.class)
    public void getUrls_UnexistingDir() throws IOException {

        final File rootDir = this.tempFolder.newFolder();
        Utils.getUrls(new File(rootDir, "unexisting-dir"));
    }
}
